
# qhasm: int64 rp

# qhasm: int64 qp

# qhasm: input rp

# qhasm: input qp

# qhasm:   int64 caller1

# qhasm:   int64 caller2

# qhasm:   int64 caller3

# qhasm:   int64 caller4

# qhasm:   int64 caller5

# qhasm:   int64 caller6

# qhasm:   int64 caller7

# qhasm:   caller caller1

# qhasm:   caller caller2

# qhasm:   caller caller3

# qhasm:   caller caller4

# qhasm:   caller caller5

# qhasm:   caller caller6

# qhasm:   caller caller7

# qhasm:   stack64 caller1_stack

# qhasm:   stack64 caller2_stack

# qhasm:   stack64 caller3_stack

# qhasm:   stack64 caller4_stack

# qhasm:   stack64 caller5_stack

# qhasm:   stack64 caller6_stack

# qhasm:   stack64 caller7_stack

# qhasm: int64 a0

# qhasm: int64 a1

# qhasm: int64 a2

# qhasm: int64 a3

# qhasm: int64 a4

# qhasm: stack64 a0_stack

# qhasm: stack64 a1_stack

# qhasm: stack64 a2_stack

# qhasm: stack64 a3_stack

# qhasm: stack64 a4_stack

# qhasm: int64 b0

# qhasm: int64 b1

# qhasm: int64 b2

# qhasm: int64 b3

# qhasm: int64 b4

# qhasm: stack64 b0_stack

# qhasm: stack64 b1_stack

# qhasm: stack64 b2_stack

# qhasm: stack64 b3_stack

# qhasm: stack64 b4_stack

# qhasm: int64 c0

# qhasm: int64 c1

# qhasm: int64 c2

# qhasm: int64 c3

# qhasm: int64 c4

# qhasm: stack64 c0_stack

# qhasm: stack64 c1_stack

# qhasm: stack64 c2_stack

# qhasm: stack64 c3_stack

# qhasm: stack64 c4_stack

# qhasm: int64 d0

# qhasm: int64 d1

# qhasm: int64 d2

# qhasm: int64 d3

# qhasm: int64 d4

# qhasm: stack64 d0_stack

# qhasm: stack64 d1_stack

# qhasm: stack64 d2_stack

# qhasm: stack64 d3_stack

# qhasm: stack64 d4_stack

# qhasm: int64 e0

# qhasm: int64 e1

# qhasm: int64 e2

# qhasm: int64 e3

# qhasm: int64 e4

# qhasm: stack64 e0_stack

# qhasm: stack64 e1_stack

# qhasm: stack64 e2_stack

# qhasm: stack64 e3_stack

# qhasm: stack64 e4_stack

# qhasm: int64 f0

# qhasm: int64 f1

# qhasm: int64 f2

# qhasm: int64 f3

# qhasm: int64 f4

# qhasm: stack64 f0_stack

# qhasm: stack64 f1_stack

# qhasm: stack64 f2_stack

# qhasm: stack64 f3_stack

# qhasm: stack64 f4_stack

# qhasm: int64 g0

# qhasm: int64 g1

# qhasm: int64 g2

# qhasm: int64 g3

# qhasm: int64 g4

# qhasm: stack64 g0_stack

# qhasm: stack64 g1_stack

# qhasm: stack64 g2_stack

# qhasm: stack64 g3_stack

# qhasm: stack64 g4_stack

# qhasm: int64 h0

# qhasm: int64 h1

# qhasm: int64 h2

# qhasm: int64 h3

# qhasm: int64 h4

# qhasm: stack64 h0_stack

# qhasm: stack64 h1_stack

# qhasm: stack64 h2_stack

# qhasm: stack64 h3_stack

# qhasm: stack64 h4_stack

# qhasm: int64 qt0

# qhasm: int64 qt1

# qhasm: int64 qt2

# qhasm: int64 qt3

# qhasm: int64 qt4

# qhasm: stack64 qt0_stack

# qhasm: stack64 qt1_stack

# qhasm: stack64 qt2_stack

# qhasm: stack64 qt3_stack

# qhasm: stack64 qt4_stack

# qhasm: int64 t10

# qhasm: int64 t11

# qhasm: int64 t12

# qhasm: int64 t13

# qhasm: int64 t14

# qhasm: stack64 t10_stack

# qhasm: stack64 t11_stack

# qhasm: stack64 t12_stack

# qhasm: stack64 t13_stack

# qhasm: stack64 t14_stack

# qhasm: int64 t20

# qhasm: int64 t21

# qhasm: int64 t22

# qhasm: int64 t23

# qhasm: int64 t24

# qhasm: stack64 t20_stack

# qhasm: stack64 t21_stack

# qhasm: stack64 t22_stack

# qhasm: stack64 t23_stack

# qhasm: stack64 t24_stack

# qhasm: int64 rx0

# qhasm: int64 rx1

# qhasm: int64 rx2

# qhasm: int64 rx3

# qhasm: int64 rx4

# qhasm: int64 ry0

# qhasm: int64 ry1

# qhasm: int64 ry2

# qhasm: int64 ry3

# qhasm: int64 ry4

# qhasm: int64 rz0

# qhasm: int64 rz1

# qhasm: int64 rz2

# qhasm: int64 rz3

# qhasm: int64 rz4

# qhasm: int64 rt0

# qhasm: int64 rt1

# qhasm: int64 rt2

# qhasm: int64 rt3

# qhasm: int64 rt4

# qhasm: int64 mulr01

# qhasm: int64 mulr11

# qhasm: int64 mulr21

# qhasm: int64 mulr31

# qhasm: int64 mulr41

# qhasm: int64 mulrax

# qhasm: int64 mulrdx

# qhasm: int64 mult

# qhasm: int64 mulredmask

# qhasm: stack64 mulx219_stack

# qhasm: stack64 mulx319_stack

# qhasm: stack64 mulx419_stack

# qhasm: enter crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd2
.text
.p2align 5
.globl _crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd2
.globl crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd2
_crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd2:
crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd2:
mov %rsp,%r11
and $31,%r11
add $256,%r11
sub %r11,%rsp

# qhasm:   caller1_stack = caller1
# asm 1: movq <caller1=int64#9,>caller1_stack=stack64#1
# asm 2: movq <caller1=%r11,>caller1_stack=0(%rsp)
movq %r11,0(%rsp)

# qhasm:   caller2_stack = caller2
# asm 1: movq <caller2=int64#10,>caller2_stack=stack64#2
# asm 2: movq <caller2=%r12,>caller2_stack=8(%rsp)
movq %r12,8(%rsp)

# qhasm:   caller3_stack = caller3
# asm 1: movq <caller3=int64#11,>caller3_stack=stack64#3
# asm 2: movq <caller3=%r13,>caller3_stack=16(%rsp)
movq %r13,16(%rsp)

# qhasm:   caller4_stack = caller4
# asm 1: movq <caller4=int64#12,>caller4_stack=stack64#4
# asm 2: movq <caller4=%r14,>caller4_stack=24(%rsp)
movq %r14,24(%rsp)

# qhasm:   caller5_stack = caller5
# asm 1: movq <caller5=int64#13,>caller5_stack=stack64#5
# asm 2: movq <caller5=%r15,>caller5_stack=32(%rsp)
movq %r15,32(%rsp)

# qhasm:   caller6_stack = caller6
# asm 1: movq <caller6=int64#14,>caller6_stack=stack64#6
# asm 2: movq <caller6=%rbx,>caller6_stack=40(%rsp)
movq %rbx,40(%rsp)

# qhasm:   caller7_stack = caller7
# asm 1: movq <caller7=int64#15,>caller7_stack=stack64#7
# asm 2: movq <caller7=%rbp,>caller7_stack=48(%rsp)
movq %rbp,48(%rsp)

# qhasm: a0 = *(uint64 *)(rp + 40)
# asm 1: movq   40(<rp=int64#1),>a0=int64#3
# asm 2: movq   40(<rp=%rdi),>a0=%rdx
movq   40(%rdi),%rdx

# qhasm: a1 = *(uint64 *)(rp + 48)
# asm 1: movq   48(<rp=int64#1),>a1=int64#4
# asm 2: movq   48(<rp=%rdi),>a1=%rcx
movq   48(%rdi),%rcx

# qhasm: a2 = *(uint64 *)(rp + 56)
# asm 1: movq   56(<rp=int64#1),>a2=int64#5
# asm 2: movq   56(<rp=%rdi),>a2=%r8
movq   56(%rdi),%r8

# qhasm: a3 = *(uint64 *)(rp + 64)
# asm 1: movq   64(<rp=int64#1),>a3=int64#6
# asm 2: movq   64(<rp=%rdi),>a3=%r9
movq   64(%rdi),%r9

# qhasm: a4 = *(uint64 *)(rp + 72)
# asm 1: movq   72(<rp=int64#1),>a4=int64#7
# asm 2: movq   72(<rp=%rdi),>a4=%rax
movq   72(%rdi),%rax

# qhasm: b0 = a0
# asm 1: mov  <a0=int64#3,>b0=int64#8
# asm 2: mov  <a0=%rdx,>b0=%r10
mov  %rdx,%r10

# qhasm: b1 = a1
# asm 1: mov  <a1=int64#4,>b1=int64#9
# asm 2: mov  <a1=%rcx,>b1=%r11
mov  %rcx,%r11

# qhasm: b2 = a2
# asm 1: mov  <a2=int64#5,>b2=int64#10
# asm 2: mov  <a2=%r8,>b2=%r12
mov  %r8,%r12

# qhasm: b3 = a3
# asm 1: mov  <a3=int64#6,>b3=int64#11
# asm 2: mov  <a3=%r9,>b3=%r13
mov  %r9,%r13

# qhasm: b4 = a4
# asm 1: mov  <a4=int64#7,>b4=int64#12
# asm 2: mov  <a4=%rax,>b4=%r14
mov  %rax,%r14

# qhasm: a0 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P0
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<a0=int64#3
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<a0=%rdx
add  crypto_sign_ed25519_amd64_51_30k_batch_2P0(%rip),%rdx

# qhasm: a1 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a1=int64#4
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a1=%rcx
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%rcx

# qhasm: a2 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a2=int64#5
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a2=%r8
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r8

# qhasm: a3 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a3=int64#6
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a3=%r9
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r9

# qhasm: a4 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a4=int64#7
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a4=%rax
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%rax

# qhasm: b0 += *(uint64 *) (rp + 0)
# asm 1: addq 0(<rp=int64#1),<b0=int64#8
# asm 2: addq 0(<rp=%rdi),<b0=%r10
addq 0(%rdi),%r10

# qhasm: b1 += *(uint64 *) (rp + 8)
# asm 1: addq 8(<rp=int64#1),<b1=int64#9
# asm 2: addq 8(<rp=%rdi),<b1=%r11
addq 8(%rdi),%r11

# qhasm: b2 += *(uint64 *) (rp + 16)
# asm 1: addq 16(<rp=int64#1),<b2=int64#10
# asm 2: addq 16(<rp=%rdi),<b2=%r12
addq 16(%rdi),%r12

# qhasm: b3 += *(uint64 *) (rp + 24)
# asm 1: addq 24(<rp=int64#1),<b3=int64#11
# asm 2: addq 24(<rp=%rdi),<b3=%r13
addq 24(%rdi),%r13

# qhasm: b4 += *(uint64 *) (rp + 32)
# asm 1: addq 32(<rp=int64#1),<b4=int64#12
# asm 2: addq 32(<rp=%rdi),<b4=%r14
addq 32(%rdi),%r14

# qhasm: a0 -= *(uint64 *) (rp + 0)
# asm 1: subq 0(<rp=int64#1),<a0=int64#3
# asm 2: subq 0(<rp=%rdi),<a0=%rdx
subq 0(%rdi),%rdx

# qhasm: a1 -= *(uint64 *) (rp + 8)
# asm 1: subq 8(<rp=int64#1),<a1=int64#4
# asm 2: subq 8(<rp=%rdi),<a1=%rcx
subq 8(%rdi),%rcx

# qhasm: a2 -= *(uint64 *) (rp + 16)
# asm 1: subq 16(<rp=int64#1),<a2=int64#5
# asm 2: subq 16(<rp=%rdi),<a2=%r8
subq 16(%rdi),%r8

# qhasm: a3 -= *(uint64 *) (rp + 24)
# asm 1: subq 24(<rp=int64#1),<a3=int64#6
# asm 2: subq 24(<rp=%rdi),<a3=%r9
subq 24(%rdi),%r9

# qhasm: a4 -= *(uint64 *) (rp + 32)
# asm 1: subq 32(<rp=int64#1),<a4=int64#7
# asm 2: subq 32(<rp=%rdi),<a4=%rax
subq 32(%rdi),%rax

# qhasm: a0_stack = a0
# asm 1: movq <a0=int64#3,>a0_stack=stack64#8
# asm 2: movq <a0=%rdx,>a0_stack=56(%rsp)
movq %rdx,56(%rsp)

# qhasm: a1_stack = a1
# asm 1: movq <a1=int64#4,>a1_stack=stack64#9
# asm 2: movq <a1=%rcx,>a1_stack=64(%rsp)
movq %rcx,64(%rsp)

# qhasm: a2_stack = a2
# asm 1: movq <a2=int64#5,>a2_stack=stack64#10
# asm 2: movq <a2=%r8,>a2_stack=72(%rsp)
movq %r8,72(%rsp)

# qhasm: a3_stack = a3
# asm 1: movq <a3=int64#6,>a3_stack=stack64#11
# asm 2: movq <a3=%r9,>a3_stack=80(%rsp)
movq %r9,80(%rsp)

# qhasm: a4_stack = a4
# asm 1: movq <a4=int64#7,>a4_stack=stack64#12
# asm 2: movq <a4=%rax,>a4_stack=88(%rsp)
movq %rax,88(%rsp)

# qhasm: b0_stack = b0
# asm 1: movq <b0=int64#8,>b0_stack=stack64#13
# asm 2: movq <b0=%r10,>b0_stack=96(%rsp)
movq %r10,96(%rsp)

# qhasm: b1_stack = b1
# asm 1: movq <b1=int64#9,>b1_stack=stack64#14
# asm 2: movq <b1=%r11,>b1_stack=104(%rsp)
movq %r11,104(%rsp)

# qhasm: b2_stack = b2
# asm 1: movq <b2=int64#10,>b2_stack=stack64#15
# asm 2: movq <b2=%r12,>b2_stack=112(%rsp)
movq %r12,112(%rsp)

# qhasm: b3_stack = b3
# asm 1: movq <b3=int64#11,>b3_stack=stack64#16
# asm 2: movq <b3=%r13,>b3_stack=120(%rsp)
movq %r13,120(%rsp)

# qhasm: b4_stack = b4
# asm 1: movq <b4=int64#12,>b4_stack=stack64#17
# asm 2: movq <b4=%r14,>b4_stack=128(%rsp)
movq %r14,128(%rsp)

# qhasm:   mulrax = a3_stack
# asm 1: movq <a3_stack=stack64#11,>mulrax=int64#3
# asm 2: movq <a3_stack=80(%rsp),>mulrax=%rdx
movq 80(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#18
# asm 2: movq <mulrax=%rax,>mulx319_stack=136(%rsp)
movq %rax,136(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#2)
# asm 2: mulq  16(<qp=%rsi)
mulq  16(%rsi)

# qhasm:   a0 = mulrax
# asm 1: mov  <mulrax=int64#7,>a0=int64#4
# asm 2: mov  <mulrax=%rax,>a0=%rcx
mov  %rax,%rcx

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#5
# asm 2: mov  <mulrdx=%rdx,>mulr01=%r8
mov  %rdx,%r8

# qhasm:   mulrax = a4_stack
# asm 1: movq <a4_stack=stack64#12,>mulrax=int64#3
# asm 2: movq <a4_stack=88(%rsp),>mulrax=%rdx
movq 88(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#19
# asm 2: movq <mulrax=%rax,>mulx419_stack=144(%rsp)
movq %rax,144(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#2)
# asm 2: mulq  8(<qp=%rsi)
mulq  8(%rsi)

# qhasm:   carry? a0 += mulrax
# asm 1: add  <mulrax=int64#7,<a0=int64#4
# asm 2: add  <mulrax=%rax,<a0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#2)
# asm 2: mulq  0(<qp=%rsi)
mulq  0(%rsi)

# qhasm:   carry? a0 += mulrax
# asm 1: add  <mulrax=int64#7,<a0=int64#4
# asm 2: add  <mulrax=%rax,<a0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#2)
# asm 2: mulq  8(<qp=%rsi)
mulq  8(%rsi)

# qhasm:   a1 = mulrax
# asm 1: mov  <mulrax=int64#7,>a1=int64#6
# asm 2: mov  <mulrax=%rax,>a1=%r9
mov  %rax,%r9

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#8
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r10
mov  %rdx,%r10

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#2)
# asm 2: mulq  16(<qp=%rsi)
mulq  16(%rsi)

# qhasm:   a2 = mulrax
# asm 1: mov  <mulrax=int64#7,>a2=int64#9
# asm 2: mov  <mulrax=%rax,>a2=%r11
mov  %rax,%r11

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#10
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r12
mov  %rdx,%r12

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#2)
# asm 2: mulq  24(<qp=%rsi)
mulq  24(%rsi)

# qhasm:   a3 = mulrax
# asm 1: mov  <mulrax=int64#7,>a3=int64#11
# asm 2: mov  <mulrax=%rax,>a3=%r13
mov  %rax,%r13

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#12
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r14
mov  %rdx,%r14

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#2)
# asm 2: mulq  32(<qp=%rsi)
mulq  32(%rsi)

# qhasm:   a4 = mulrax
# asm 1: mov  <mulrax=int64#7,>a4=int64#13
# asm 2: mov  <mulrax=%rax,>a4=%r15
mov  %rax,%r15

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#14
# asm 2: mov  <mulrdx=%rdx,>mulr41=%rbx
mov  %rdx,%rbx

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#2)
# asm 2: mulq  0(<qp=%rsi)
mulq  0(%rsi)

# qhasm:   carry? a1 += mulrax
# asm 1: add  <mulrax=int64#7,<a1=int64#6
# asm 2: add  <mulrax=%rax,<a1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#2)
# asm 2: mulq  8(<qp=%rsi)
mulq  8(%rsi)

# qhasm:   carry? a2 += mulrax
# asm 1: add  <mulrax=int64#7,<a2=int64#9
# asm 2: add  <mulrax=%rax,<a2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#2)
# asm 2: mulq  16(<qp=%rsi)
mulq  16(%rsi)

# qhasm:   carry? a3 += mulrax
# asm 1: add  <mulrax=int64#7,<a3=int64#11
# asm 2: add  <mulrax=%rax,<a3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#2)
# asm 2: mulq  24(<qp=%rsi)
mulq  24(%rsi)

# qhasm:   carry? a4 += mulrax
# asm 1: add  <mulrax=int64#7,<a4=int64#13
# asm 2: add  <mulrax=%rax,<a4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#3
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rdx
movq 64(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#2)
# asm 2: mulq  32(<qp=%rsi)
mulq  32(%rsi)

# qhasm:   carry? a0 += mulrax
# asm 1: add  <mulrax=int64#7,<a0=int64#4
# asm 2: add  <mulrax=%rax,<a0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#7
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rax
movq 72(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#2)
# asm 2: mulq  0(<qp=%rsi)
mulq  0(%rsi)

# qhasm:   carry? a2 += mulrax
# asm 1: add  <mulrax=int64#7,<a2=int64#9
# asm 2: add  <mulrax=%rax,<a2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#7
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rax
movq 72(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#2)
# asm 2: mulq  8(<qp=%rsi)
mulq  8(%rsi)

# qhasm:   carry? a3 += mulrax
# asm 1: add  <mulrax=int64#7,<a3=int64#11
# asm 2: add  <mulrax=%rax,<a3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#7
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rax
movq 72(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#2)
# asm 2: mulq  16(<qp=%rsi)
mulq  16(%rsi)

# qhasm:   carry? a4 += mulrax
# asm 1: add  <mulrax=int64#7,<a4=int64#13
# asm 2: add  <mulrax=%rax,<a4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#3
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rdx
movq 72(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#2)
# asm 2: mulq  24(<qp=%rsi)
mulq  24(%rsi)

# qhasm:   carry? a0 += mulrax
# asm 1: add  <mulrax=int64#7,<a0=int64#4
# asm 2: add  <mulrax=%rax,<a0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#3
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rdx
movq 72(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#2)
# asm 2: mulq  32(<qp=%rsi)
mulq  32(%rsi)

# qhasm:   carry? a1 += mulrax
# asm 1: add  <mulrax=int64#7,<a1=int64#6
# asm 2: add  <mulrax=%rax,<a1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = a3_stack
# asm 1: movq <a3_stack=stack64#11,>mulrax=int64#7
# asm 2: movq <a3_stack=80(%rsp),>mulrax=%rax
movq 80(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#2)
# asm 2: mulq  0(<qp=%rsi)
mulq  0(%rsi)

# qhasm:   carry? a3 += mulrax
# asm 1: add  <mulrax=int64#7,<a3=int64#11
# asm 2: add  <mulrax=%rax,<a3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulrax = a3_stack
# asm 1: movq <a3_stack=stack64#11,>mulrax=int64#7
# asm 2: movq <a3_stack=80(%rsp),>mulrax=%rax
movq 80(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#2)
# asm 2: mulq  8(<qp=%rsi)
mulq  8(%rsi)

# qhasm:   carry? a4 += mulrax
# asm 1: add  <mulrax=int64#7,<a4=int64#13
# asm 2: add  <mulrax=%rax,<a4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#2)
# asm 2: mulq  24(<qp=%rsi)
mulq  24(%rsi)

# qhasm:   carry? a1 += mulrax
# asm 1: add  <mulrax=int64#7,<a1=int64#6
# asm 2: add  <mulrax=%rax,<a1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#2)
# asm 2: mulq  32(<qp=%rsi)
mulq  32(%rsi)

# qhasm:   carry? a2 += mulrax
# asm 1: add  <mulrax=int64#7,<a2=int64#9
# asm 2: add  <mulrax=%rax,<a2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = a4_stack
# asm 1: movq <a4_stack=stack64#12,>mulrax=int64#7
# asm 2: movq <a4_stack=88(%rsp),>mulrax=%rax
movq 88(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#2)
# asm 2: mulq  0(<qp=%rsi)
mulq  0(%rsi)

# qhasm:   carry? a4 += mulrax
# asm 1: add  <mulrax=int64#7,<a4=int64#13
# asm 2: add  <mulrax=%rax,<a4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#2)
# asm 2: mulq  16(<qp=%rsi)
mulq  16(%rsi)

# qhasm:   carry? a1 += mulrax
# asm 1: add  <mulrax=int64#7,<a1=int64#6
# asm 2: add  <mulrax=%rax,<a1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#2)
# asm 2: mulq  24(<qp=%rsi)
mulq  24(%rsi)

# qhasm:   carry? a2 += mulrax
# asm 1: add  <mulrax=int64#7,<a2=int64#9
# asm 2: add  <mulrax=%rax,<a2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#2)
# asm 2: mulq  32(<qp=%rsi)
mulq  32(%rsi)

# qhasm:   carry? a3 += mulrax
# asm 1: add  <mulrax=int64#7,<a3=int64#11
# asm 2: add  <mulrax=%rax,<a3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#3
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rdx
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rdx

# qhasm:   mulr01 = (mulr01.a0) << 13
# asm 1: shld $13,<a0=int64#4,<mulr01=int64#5
# asm 2: shld $13,<a0=%rcx,<mulr01=%r8
shld $13,%rcx,%r8

# qhasm:   a0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a0=int64#4
# asm 2: and  <mulredmask=%rdx,<a0=%rcx
and  %rdx,%rcx

# qhasm:   mulr11 = (mulr11.a1) << 13
# asm 1: shld $13,<a1=int64#6,<mulr11=int64#8
# asm 2: shld $13,<a1=%r9,<mulr11=%r10
shld $13,%r9,%r10

# qhasm:   a1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a1=int64#6
# asm 2: and  <mulredmask=%rdx,<a1=%r9
and  %rdx,%r9

# qhasm:   a1 += mulr01
# asm 1: add  <mulr01=int64#5,<a1=int64#6
# asm 2: add  <mulr01=%r8,<a1=%r9
add  %r8,%r9

# qhasm:   mulr21 = (mulr21.a2) << 13
# asm 1: shld $13,<a2=int64#9,<mulr21=int64#10
# asm 2: shld $13,<a2=%r11,<mulr21=%r12
shld $13,%r11,%r12

# qhasm:   a2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a2=int64#9
# asm 2: and  <mulredmask=%rdx,<a2=%r11
and  %rdx,%r11

# qhasm:   a2 += mulr11
# asm 1: add  <mulr11=int64#8,<a2=int64#9
# asm 2: add  <mulr11=%r10,<a2=%r11
add  %r10,%r11

# qhasm:   mulr31 = (mulr31.a3) << 13
# asm 1: shld $13,<a3=int64#11,<mulr31=int64#12
# asm 2: shld $13,<a3=%r13,<mulr31=%r14
shld $13,%r13,%r14

# qhasm:   a3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a3=int64#11
# asm 2: and  <mulredmask=%rdx,<a3=%r13
and  %rdx,%r13

# qhasm:   a3 += mulr21
# asm 1: add  <mulr21=int64#10,<a3=int64#11
# asm 2: add  <mulr21=%r12,<a3=%r13
add  %r12,%r13

# qhasm:   mulr41 = (mulr41.a4) << 13
# asm 1: shld $13,<a4=int64#13,<mulr41=int64#14
# asm 2: shld $13,<a4=%r15,<mulr41=%rbx
shld $13,%r15,%rbx

# qhasm:   a4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a4=int64#13
# asm 2: and  <mulredmask=%rdx,<a4=%r15
and  %rdx,%r15

# qhasm:   a4 += mulr31
# asm 1: add  <mulr31=int64#12,<a4=int64#13
# asm 2: add  <mulr31=%r14,<a4=%r15
add  %r14,%r15

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#14,>mulr41=int64#5
# asm 2: imulq  $19,<mulr41=%rbx,>mulr41=%r8
imulq  $19,%rbx,%r8

# qhasm:   a0 += mulr41
# asm 1: add  <mulr41=int64#5,<a0=int64#4
# asm 2: add  <mulr41=%r8,<a0=%rcx
add  %r8,%rcx

# qhasm:   mult = a0
# asm 1: mov  <a0=int64#4,>mult=int64#5
# asm 2: mov  <a0=%rcx,>mult=%r8
mov  %rcx,%r8

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   mult += a1
# asm 1: add  <a1=int64#6,<mult=int64#5
# asm 2: add  <a1=%r9,<mult=%r8
add  %r9,%r8

# qhasm:   a1 = mult
# asm 1: mov  <mult=int64#5,>a1=int64#6
# asm 2: mov  <mult=%r8,>a1=%r9
mov  %r8,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   a0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a0=int64#4
# asm 2: and  <mulredmask=%rdx,<a0=%rcx
and  %rdx,%rcx

# qhasm:   mult += a2
# asm 1: add  <a2=int64#9,<mult=int64#5
# asm 2: add  <a2=%r11,<mult=%r8
add  %r11,%r8

# qhasm:   a2 = mult
# asm 1: mov  <mult=int64#5,>a2=int64#7
# asm 2: mov  <mult=%r8,>a2=%rax
mov  %r8,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   a1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a1=int64#6
# asm 2: and  <mulredmask=%rdx,<a1=%r9
and  %rdx,%r9

# qhasm:   mult += a3
# asm 1: add  <a3=int64#11,<mult=int64#5
# asm 2: add  <a3=%r13,<mult=%r8
add  %r13,%r8

# qhasm:   a3 = mult
# asm 1: mov  <mult=int64#5,>a3=int64#8
# asm 2: mov  <mult=%r8,>a3=%r10
mov  %r8,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   a2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a2=int64#7
# asm 2: and  <mulredmask=%rdx,<a2=%rax
and  %rdx,%rax

# qhasm:   mult += a4
# asm 1: add  <a4=int64#13,<mult=int64#5
# asm 2: add  <a4=%r15,<mult=%r8
add  %r15,%r8

# qhasm:   a4 = mult
# asm 1: mov  <mult=int64#5,>a4=int64#9
# asm 2: mov  <mult=%r8,>a4=%r11
mov  %r8,%r11

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   a3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a3=int64#8
# asm 2: and  <mulredmask=%rdx,<a3=%r10
and  %rdx,%r10

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#5,>mult=int64#5
# asm 2: imulq  $19,<mult=%r8,>mult=%r8
imulq  $19,%r8,%r8

# qhasm:   a0 += mult
# asm 1: add  <mult=int64#5,<a0=int64#4
# asm 2: add  <mult=%r8,<a0=%rcx
add  %r8,%rcx

# qhasm:   a4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a4=int64#9
# asm 2: and  <mulredmask=%rdx,<a4=%r11
and  %rdx,%r11

# qhasm: a0_stack = a0
# asm 1: movq <a0=int64#4,>a0_stack=stack64#8
# asm 2: movq <a0=%rcx,>a0_stack=56(%rsp)
movq %rcx,56(%rsp)

# qhasm: a1_stack = a1
# asm 1: movq <a1=int64#6,>a1_stack=stack64#9
# asm 2: movq <a1=%r9,>a1_stack=64(%rsp)
movq %r9,64(%rsp)

# qhasm: a2_stack = a2
# asm 1: movq <a2=int64#7,>a2_stack=stack64#10
# asm 2: movq <a2=%rax,>a2_stack=72(%rsp)
movq %rax,72(%rsp)

# qhasm: a3_stack = a3
# asm 1: movq <a3=int64#8,>a3_stack=stack64#11
# asm 2: movq <a3=%r10,>a3_stack=80(%rsp)
movq %r10,80(%rsp)

# qhasm: a4_stack = a4
# asm 1: movq <a4=int64#9,>a4_stack=stack64#12
# asm 2: movq <a4=%r11,>a4_stack=88(%rsp)
movq %r11,88(%rsp)

# qhasm:   mulrax = b3_stack
# asm 1: movq <b3_stack=stack64#16,>mulrax=int64#3
# asm 2: movq <b3_stack=120(%rsp),>mulrax=%rdx
movq 120(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#18
# asm 2: movq <mulrax=%rax,>mulx319_stack=136(%rsp)
movq %rax,136(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#2)
# asm 2: mulq  56(<qp=%rsi)
mulq  56(%rsi)

# qhasm:   e0 = mulrax
# asm 1: mov  <mulrax=int64#7,>e0=int64#4
# asm 2: mov  <mulrax=%rax,>e0=%rcx
mov  %rax,%rcx

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#5
# asm 2: mov  <mulrdx=%rdx,>mulr01=%r8
mov  %rdx,%r8

# qhasm:   mulrax = b4_stack
# asm 1: movq <b4_stack=stack64#17,>mulrax=int64#3
# asm 2: movq <b4_stack=128(%rsp),>mulrax=%rdx
movq 128(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#19
# asm 2: movq <mulrax=%rax,>mulx419_stack=144(%rsp)
movq %rax,144(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#2)
# asm 2: mulq  48(<qp=%rsi)
mulq  48(%rsi)

# qhasm:   carry? e0 += mulrax
# asm 1: add  <mulrax=int64#7,<e0=int64#4
# asm 2: add  <mulrax=%rax,<e0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#2)
# asm 2: mulq  40(<qp=%rsi)
mulq  40(%rsi)

# qhasm:   carry? e0 += mulrax
# asm 1: add  <mulrax=int64#7,<e0=int64#4
# asm 2: add  <mulrax=%rax,<e0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#2)
# asm 2: mulq  48(<qp=%rsi)
mulq  48(%rsi)

# qhasm:   e1 = mulrax
# asm 1: mov  <mulrax=int64#7,>e1=int64#6
# asm 2: mov  <mulrax=%rax,>e1=%r9
mov  %rax,%r9

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#8
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r10
mov  %rdx,%r10

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#2)
# asm 2: mulq  56(<qp=%rsi)
mulq  56(%rsi)

# qhasm:   e2 = mulrax
# asm 1: mov  <mulrax=int64#7,>e2=int64#9
# asm 2: mov  <mulrax=%rax,>e2=%r11
mov  %rax,%r11

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#10
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r12
mov  %rdx,%r12

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#2)
# asm 2: mulq  64(<qp=%rsi)
mulq  64(%rsi)

# qhasm:   e3 = mulrax
# asm 1: mov  <mulrax=int64#7,>e3=int64#11
# asm 2: mov  <mulrax=%rax,>e3=%r13
mov  %rax,%r13

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#12
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r14
mov  %rdx,%r14

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#2)
# asm 2: mulq  72(<qp=%rsi)
mulq  72(%rsi)

# qhasm:   e4 = mulrax
# asm 1: mov  <mulrax=int64#7,>e4=int64#13
# asm 2: mov  <mulrax=%rax,>e4=%r15
mov  %rax,%r15

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#14
# asm 2: mov  <mulrdx=%rdx,>mulr41=%rbx
mov  %rdx,%rbx

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#2)
# asm 2: mulq  40(<qp=%rsi)
mulq  40(%rsi)

# qhasm:   carry? e1 += mulrax
# asm 1: add  <mulrax=int64#7,<e1=int64#6
# asm 2: add  <mulrax=%rax,<e1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#2)
# asm 2: mulq  48(<qp=%rsi)
mulq  48(%rsi)

# qhasm:   carry? e2 += mulrax
# asm 1: add  <mulrax=int64#7,<e2=int64#9
# asm 2: add  <mulrax=%rax,<e2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#2)
# asm 2: mulq  56(<qp=%rsi)
mulq  56(%rsi)

# qhasm:   carry? e3 += mulrax
# asm 1: add  <mulrax=int64#7,<e3=int64#11
# asm 2: add  <mulrax=%rax,<e3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#2)
# asm 2: mulq  64(<qp=%rsi)
mulq  64(%rsi)

# qhasm:   carry? e4 += mulrax
# asm 1: add  <mulrax=int64#7,<e4=int64#13
# asm 2: add  <mulrax=%rax,<e4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#3
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rdx
movq 104(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#2)
# asm 2: mulq  72(<qp=%rsi)
mulq  72(%rsi)

# qhasm:   carry? e0 += mulrax
# asm 1: add  <mulrax=int64#7,<e0=int64#4
# asm 2: add  <mulrax=%rax,<e0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#2)
# asm 2: mulq  40(<qp=%rsi)
mulq  40(%rsi)

# qhasm:   carry? e2 += mulrax
# asm 1: add  <mulrax=int64#7,<e2=int64#9
# asm 2: add  <mulrax=%rax,<e2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#2)
# asm 2: mulq  48(<qp=%rsi)
mulq  48(%rsi)

# qhasm:   carry? e3 += mulrax
# asm 1: add  <mulrax=int64#7,<e3=int64#11
# asm 2: add  <mulrax=%rax,<e3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#2)
# asm 2: mulq  56(<qp=%rsi)
mulq  56(%rsi)

# qhasm:   carry? e4 += mulrax
# asm 1: add  <mulrax=int64#7,<e4=int64#13
# asm 2: add  <mulrax=%rax,<e4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#3
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rdx
movq 112(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#2)
# asm 2: mulq  64(<qp=%rsi)
mulq  64(%rsi)

# qhasm:   carry? e0 += mulrax
# asm 1: add  <mulrax=int64#7,<e0=int64#4
# asm 2: add  <mulrax=%rax,<e0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#3
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rdx
movq 112(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#2)
# asm 2: mulq  72(<qp=%rsi)
mulq  72(%rsi)

# qhasm:   carry? e1 += mulrax
# asm 1: add  <mulrax=int64#7,<e1=int64#6
# asm 2: add  <mulrax=%rax,<e1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = b3_stack
# asm 1: movq <b3_stack=stack64#16,>mulrax=int64#7
# asm 2: movq <b3_stack=120(%rsp),>mulrax=%rax
movq 120(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#2)
# asm 2: mulq  40(<qp=%rsi)
mulq  40(%rsi)

# qhasm:   carry? e3 += mulrax
# asm 1: add  <mulrax=int64#7,<e3=int64#11
# asm 2: add  <mulrax=%rax,<e3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulrax = b3_stack
# asm 1: movq <b3_stack=stack64#16,>mulrax=int64#7
# asm 2: movq <b3_stack=120(%rsp),>mulrax=%rax
movq 120(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#2)
# asm 2: mulq  48(<qp=%rsi)
mulq  48(%rsi)

# qhasm:   carry? e4 += mulrax
# asm 1: add  <mulrax=int64#7,<e4=int64#13
# asm 2: add  <mulrax=%rax,<e4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#2)
# asm 2: mulq  64(<qp=%rsi)
mulq  64(%rsi)

# qhasm:   carry? e1 += mulrax
# asm 1: add  <mulrax=int64#7,<e1=int64#6
# asm 2: add  <mulrax=%rax,<e1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#2)
# asm 2: mulq  72(<qp=%rsi)
mulq  72(%rsi)

# qhasm:   carry? e2 += mulrax
# asm 1: add  <mulrax=int64#7,<e2=int64#9
# asm 2: add  <mulrax=%rax,<e2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = b4_stack
# asm 1: movq <b4_stack=stack64#17,>mulrax=int64#7
# asm 2: movq <b4_stack=128(%rsp),>mulrax=%rax
movq 128(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#2)
# asm 2: mulq  40(<qp=%rsi)
mulq  40(%rsi)

# qhasm:   carry? e4 += mulrax
# asm 1: add  <mulrax=int64#7,<e4=int64#13
# asm 2: add  <mulrax=%rax,<e4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#2)
# asm 2: mulq  56(<qp=%rsi)
mulq  56(%rsi)

# qhasm:   carry? e1 += mulrax
# asm 1: add  <mulrax=int64#7,<e1=int64#6
# asm 2: add  <mulrax=%rax,<e1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#2)
# asm 2: mulq  64(<qp=%rsi)
mulq  64(%rsi)

# qhasm:   carry? e2 += mulrax
# asm 1: add  <mulrax=int64#7,<e2=int64#9
# asm 2: add  <mulrax=%rax,<e2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#2)
# asm 2: mulq  72(<qp=%rsi)
mulq  72(%rsi)

# qhasm:   carry? e3 += mulrax
# asm 1: add  <mulrax=int64#7,<e3=int64#11
# asm 2: add  <mulrax=%rax,<e3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#3
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rdx
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rdx

# qhasm:   mulr01 = (mulr01.e0) << 13
# asm 1: shld $13,<e0=int64#4,<mulr01=int64#5
# asm 2: shld $13,<e0=%rcx,<mulr01=%r8
shld $13,%rcx,%r8

# qhasm:   e0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e0=int64#4
# asm 2: and  <mulredmask=%rdx,<e0=%rcx
and  %rdx,%rcx

# qhasm:   mulr11 = (mulr11.e1) << 13
# asm 1: shld $13,<e1=int64#6,<mulr11=int64#8
# asm 2: shld $13,<e1=%r9,<mulr11=%r10
shld $13,%r9,%r10

# qhasm:   e1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e1=int64#6
# asm 2: and  <mulredmask=%rdx,<e1=%r9
and  %rdx,%r9

# qhasm:   e1 += mulr01
# asm 1: add  <mulr01=int64#5,<e1=int64#6
# asm 2: add  <mulr01=%r8,<e1=%r9
add  %r8,%r9

# qhasm:   mulr21 = (mulr21.e2) << 13
# asm 1: shld $13,<e2=int64#9,<mulr21=int64#10
# asm 2: shld $13,<e2=%r11,<mulr21=%r12
shld $13,%r11,%r12

# qhasm:   e2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e2=int64#9
# asm 2: and  <mulredmask=%rdx,<e2=%r11
and  %rdx,%r11

# qhasm:   e2 += mulr11
# asm 1: add  <mulr11=int64#8,<e2=int64#9
# asm 2: add  <mulr11=%r10,<e2=%r11
add  %r10,%r11

# qhasm:   mulr31 = (mulr31.e3) << 13
# asm 1: shld $13,<e3=int64#11,<mulr31=int64#12
# asm 2: shld $13,<e3=%r13,<mulr31=%r14
shld $13,%r13,%r14

# qhasm:   e3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e3=int64#11
# asm 2: and  <mulredmask=%rdx,<e3=%r13
and  %rdx,%r13

# qhasm:   e3 += mulr21
# asm 1: add  <mulr21=int64#10,<e3=int64#11
# asm 2: add  <mulr21=%r12,<e3=%r13
add  %r12,%r13

# qhasm:   mulr41 = (mulr41.e4) << 13
# asm 1: shld $13,<e4=int64#13,<mulr41=int64#14
# asm 2: shld $13,<e4=%r15,<mulr41=%rbx
shld $13,%r15,%rbx

# qhasm:   e4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e4=int64#13
# asm 2: and  <mulredmask=%rdx,<e4=%r15
and  %rdx,%r15

# qhasm:   e4 += mulr31
# asm 1: add  <mulr31=int64#12,<e4=int64#13
# asm 2: add  <mulr31=%r14,<e4=%r15
add  %r14,%r15

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#14,>mulr41=int64#5
# asm 2: imulq  $19,<mulr41=%rbx,>mulr41=%r8
imulq  $19,%rbx,%r8

# qhasm:   e0 += mulr41
# asm 1: add  <mulr41=int64#5,<e0=int64#4
# asm 2: add  <mulr41=%r8,<e0=%rcx
add  %r8,%rcx

# qhasm:   mult = e0
# asm 1: mov  <e0=int64#4,>mult=int64#5
# asm 2: mov  <e0=%rcx,>mult=%r8
mov  %rcx,%r8

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   mult += e1
# asm 1: add  <e1=int64#6,<mult=int64#5
# asm 2: add  <e1=%r9,<mult=%r8
add  %r9,%r8

# qhasm:   e1 = mult
# asm 1: mov  <mult=int64#5,>e1=int64#6
# asm 2: mov  <mult=%r8,>e1=%r9
mov  %r8,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   e0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e0=int64#4
# asm 2: and  <mulredmask=%rdx,<e0=%rcx
and  %rdx,%rcx

# qhasm:   mult += e2
# asm 1: add  <e2=int64#9,<mult=int64#5
# asm 2: add  <e2=%r11,<mult=%r8
add  %r11,%r8

# qhasm:   e2 = mult
# asm 1: mov  <mult=int64#5,>e2=int64#7
# asm 2: mov  <mult=%r8,>e2=%rax
mov  %r8,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   e1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e1=int64#6
# asm 2: and  <mulredmask=%rdx,<e1=%r9
and  %rdx,%r9

# qhasm:   mult += e3
# asm 1: add  <e3=int64#11,<mult=int64#5
# asm 2: add  <e3=%r13,<mult=%r8
add  %r13,%r8

# qhasm:   e3 = mult
# asm 1: mov  <mult=int64#5,>e3=int64#8
# asm 2: mov  <mult=%r8,>e3=%r10
mov  %r8,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   e2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e2=int64#7
# asm 2: and  <mulredmask=%rdx,<e2=%rax
and  %rdx,%rax

# qhasm:   mult += e4
# asm 1: add  <e4=int64#13,<mult=int64#5
# asm 2: add  <e4=%r15,<mult=%r8
add  %r15,%r8

# qhasm:   e4 = mult
# asm 1: mov  <mult=int64#5,>e4=int64#9
# asm 2: mov  <mult=%r8,>e4=%r11
mov  %r8,%r11

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#5
# asm 2: shr  $51,<mult=%r8
shr  $51,%r8

# qhasm:   e3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e3=int64#8
# asm 2: and  <mulredmask=%rdx,<e3=%r10
and  %rdx,%r10

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#5,>mult=int64#5
# asm 2: imulq  $19,<mult=%r8,>mult=%r8
imulq  $19,%r8,%r8

# qhasm:   e0 += mult
# asm 1: add  <mult=int64#5,<e0=int64#4
# asm 2: add  <mult=%r8,<e0=%rcx
add  %r8,%rcx

# qhasm:   e4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e4=int64#9
# asm 2: and  <mulredmask=%rdx,<e4=%r11
and  %rdx,%r11

# qhasm: h0 = e0
# asm 1: mov  <e0=int64#4,>h0=int64#3
# asm 2: mov  <e0=%rcx,>h0=%rdx
mov  %rcx,%rdx

# qhasm: h1 = e1
# asm 1: mov  <e1=int64#6,>h1=int64#5
# asm 2: mov  <e1=%r9,>h1=%r8
mov  %r9,%r8

# qhasm: h2 = e2
# asm 1: mov  <e2=int64#7,>h2=int64#10
# asm 2: mov  <e2=%rax,>h2=%r12
mov  %rax,%r12

# qhasm: h3 = e3
# asm 1: mov  <e3=int64#8,>h3=int64#11
# asm 2: mov  <e3=%r10,>h3=%r13
mov  %r10,%r13

# qhasm: h4 = e4
# asm 1: mov  <e4=int64#9,>h4=int64#12
# asm 2: mov  <e4=%r11,>h4=%r14
mov  %r11,%r14

# qhasm: e0 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P0
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<e0=int64#4
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<e0=%rcx
add  crypto_sign_ed25519_amd64_51_30k_batch_2P0(%rip),%rcx

# qhasm: e1 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e1=int64#6
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e1=%r9
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r9

# qhasm: e2 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e2=int64#7
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e2=%rax
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%rax

# qhasm: e3 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e3=int64#8
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e3=%r10
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r10

# qhasm: e4 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e4=int64#9
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e4=%r11
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r11

# qhasm: h0 += a0_stack
# asm 1: addq <a0_stack=stack64#8,<h0=int64#3
# asm 2: addq <a0_stack=56(%rsp),<h0=%rdx
addq 56(%rsp),%rdx

# qhasm: h1 += a1_stack
# asm 1: addq <a1_stack=stack64#9,<h1=int64#5
# asm 2: addq <a1_stack=64(%rsp),<h1=%r8
addq 64(%rsp),%r8

# qhasm: h2 += a2_stack
# asm 1: addq <a2_stack=stack64#10,<h2=int64#10
# asm 2: addq <a2_stack=72(%rsp),<h2=%r12
addq 72(%rsp),%r12

# qhasm: h3 += a3_stack
# asm 1: addq <a3_stack=stack64#11,<h3=int64#11
# asm 2: addq <a3_stack=80(%rsp),<h3=%r13
addq 80(%rsp),%r13

# qhasm: h4 += a4_stack
# asm 1: addq <a4_stack=stack64#12,<h4=int64#12
# asm 2: addq <a4_stack=88(%rsp),<h4=%r14
addq 88(%rsp),%r14

# qhasm: e0 -= a0_stack
# asm 1: subq <a0_stack=stack64#8,<e0=int64#4
# asm 2: subq <a0_stack=56(%rsp),<e0=%rcx
subq 56(%rsp),%rcx

# qhasm: e1 -= a1_stack
# asm 1: subq <a1_stack=stack64#9,<e1=int64#6
# asm 2: subq <a1_stack=64(%rsp),<e1=%r9
subq 64(%rsp),%r9

# qhasm: e2 -= a2_stack
# asm 1: subq <a2_stack=stack64#10,<e2=int64#7
# asm 2: subq <a2_stack=72(%rsp),<e2=%rax
subq 72(%rsp),%rax

# qhasm: e3 -= a3_stack
# asm 1: subq <a3_stack=stack64#11,<e3=int64#8
# asm 2: subq <a3_stack=80(%rsp),<e3=%r10
subq 80(%rsp),%r10

# qhasm: e4 -= a4_stack
# asm 1: subq <a4_stack=stack64#12,<e4=int64#9
# asm 2: subq <a4_stack=88(%rsp),<e4=%r11
subq 88(%rsp),%r11

# qhasm: h0_stack = h0
# asm 1: movq <h0=int64#3,>h0_stack=stack64#8
# asm 2: movq <h0=%rdx,>h0_stack=56(%rsp)
movq %rdx,56(%rsp)

# qhasm: h1_stack = h1
# asm 1: movq <h1=int64#5,>h1_stack=stack64#9
# asm 2: movq <h1=%r8,>h1_stack=64(%rsp)
movq %r8,64(%rsp)

# qhasm: h2_stack = h2
# asm 1: movq <h2=int64#10,>h2_stack=stack64#10
# asm 2: movq <h2=%r12,>h2_stack=72(%rsp)
movq %r12,72(%rsp)

# qhasm: h3_stack = h3
# asm 1: movq <h3=int64#11,>h3_stack=stack64#11
# asm 2: movq <h3=%r13,>h3_stack=80(%rsp)
movq %r13,80(%rsp)

# qhasm: h4_stack = h4
# asm 1: movq <h4=int64#12,>h4_stack=stack64#12
# asm 2: movq <h4=%r14,>h4_stack=88(%rsp)
movq %r14,88(%rsp)

# qhasm: e0_stack = e0
# asm 1: movq <e0=int64#4,>e0_stack=stack64#13
# asm 2: movq <e0=%rcx,>e0_stack=96(%rsp)
movq %rcx,96(%rsp)

# qhasm: e1_stack = e1
# asm 1: movq <e1=int64#6,>e1_stack=stack64#14
# asm 2: movq <e1=%r9,>e1_stack=104(%rsp)
movq %r9,104(%rsp)

# qhasm: e2_stack = e2
# asm 1: movq <e2=int64#7,>e2_stack=stack64#15
# asm 2: movq <e2=%rax,>e2_stack=112(%rsp)
movq %rax,112(%rsp)

# qhasm: e3_stack = e3
# asm 1: movq <e3=int64#8,>e3_stack=stack64#16
# asm 2: movq <e3=%r10,>e3_stack=120(%rsp)
movq %r10,120(%rsp)

# qhasm: e4_stack = e4
# asm 1: movq <e4=int64#9,>e4_stack=stack64#17
# asm 2: movq <e4=%r11,>e4_stack=128(%rsp)
movq %r11,128(%rsp)

# qhasm:   mulrax = *(uint64 *)(rp + 144)
# asm 1: movq   144(<rp=int64#1),>mulrax=int64#3
# asm 2: movq   144(<rp=%rdi),>mulrax=%rdx
movq   144(%rdi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#18
# asm 2: movq <mulrax=%rax,>mulx319_stack=136(%rsp)
movq %rax,136(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#2)
# asm 2: mulq  96(<qp=%rsi)
mulq  96(%rsi)

# qhasm:   c0 = mulrax
# asm 1: mov  <mulrax=int64#7,>c0=int64#4
# asm 2: mov  <mulrax=%rax,>c0=%rcx
mov  %rax,%rcx

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#5
# asm 2: mov  <mulrdx=%rdx,>mulr01=%r8
mov  %rdx,%r8

# qhasm:   mulrax = *(uint64 *)(rp + 152)
# asm 1: movq   152(<rp=int64#1),>mulrax=int64#3
# asm 2: movq   152(<rp=%rdi),>mulrax=%rdx
movq   152(%rdi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#19
# asm 2: movq <mulrax=%rax,>mulx419_stack=144(%rsp)
movq %rax,144(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#2)
# asm 2: mulq  88(<qp=%rsi)
mulq  88(%rsi)

# qhasm:   carry? c0 += mulrax
# asm 1: add  <mulrax=int64#7,<c0=int64#4
# asm 2: add  <mulrax=%rax,<c0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = *(uint64 *)(rp + 120)
# asm 1: movq   120(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   120(<rp=%rdi),>mulrax=%rax
movq   120(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#2)
# asm 2: mulq  80(<qp=%rsi)
mulq  80(%rsi)

# qhasm:   carry? c0 += mulrax
# asm 1: add  <mulrax=int64#7,<c0=int64#4
# asm 2: add  <mulrax=%rax,<c0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = *(uint64 *)(rp + 120)
# asm 1: movq   120(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   120(<rp=%rdi),>mulrax=%rax
movq   120(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#2)
# asm 2: mulq  88(<qp=%rsi)
mulq  88(%rsi)

# qhasm:   c1 = mulrax
# asm 1: mov  <mulrax=int64#7,>c1=int64#6
# asm 2: mov  <mulrax=%rax,>c1=%r9
mov  %rax,%r9

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#8
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r10
mov  %rdx,%r10

# qhasm:   mulrax = *(uint64 *)(rp + 120)
# asm 1: movq   120(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   120(<rp=%rdi),>mulrax=%rax
movq   120(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#2)
# asm 2: mulq  96(<qp=%rsi)
mulq  96(%rsi)

# qhasm:   c2 = mulrax
# asm 1: mov  <mulrax=int64#7,>c2=int64#9
# asm 2: mov  <mulrax=%rax,>c2=%r11
mov  %rax,%r11

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#10
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r12
mov  %rdx,%r12

# qhasm:   mulrax = *(uint64 *)(rp + 120)
# asm 1: movq   120(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   120(<rp=%rdi),>mulrax=%rax
movq   120(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#2)
# asm 2: mulq  104(<qp=%rsi)
mulq  104(%rsi)

# qhasm:   c3 = mulrax
# asm 1: mov  <mulrax=int64#7,>c3=int64#11
# asm 2: mov  <mulrax=%rax,>c3=%r13
mov  %rax,%r13

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#12
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r14
mov  %rdx,%r14

# qhasm:   mulrax = *(uint64 *)(rp + 120)
# asm 1: movq   120(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   120(<rp=%rdi),>mulrax=%rax
movq   120(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#2)
# asm 2: mulq  112(<qp=%rsi)
mulq  112(%rsi)

# qhasm:   c4 = mulrax
# asm 1: mov  <mulrax=int64#7,>c4=int64#13
# asm 2: mov  <mulrax=%rax,>c4=%r15
mov  %rax,%r15

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#14
# asm 2: mov  <mulrdx=%rdx,>mulr41=%rbx
mov  %rdx,%rbx

# qhasm:   mulrax = *(uint64 *)(rp + 128)
# asm 1: movq   128(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   128(<rp=%rdi),>mulrax=%rax
movq   128(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#2)
# asm 2: mulq  80(<qp=%rsi)
mulq  80(%rsi)

# qhasm:   carry? c1 += mulrax
# asm 1: add  <mulrax=int64#7,<c1=int64#6
# asm 2: add  <mulrax=%rax,<c1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = *(uint64 *)(rp + 128)
# asm 1: movq   128(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   128(<rp=%rdi),>mulrax=%rax
movq   128(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#2)
# asm 2: mulq  88(<qp=%rsi)
mulq  88(%rsi)

# qhasm:   carry? c2 += mulrax
# asm 1: add  <mulrax=int64#7,<c2=int64#9
# asm 2: add  <mulrax=%rax,<c2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = *(uint64 *)(rp + 128)
# asm 1: movq   128(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   128(<rp=%rdi),>mulrax=%rax
movq   128(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#2)
# asm 2: mulq  96(<qp=%rsi)
mulq  96(%rsi)

# qhasm:   carry? c3 += mulrax
# asm 1: add  <mulrax=int64#7,<c3=int64#11
# asm 2: add  <mulrax=%rax,<c3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulrax = *(uint64 *)(rp + 128)
# asm 1: movq   128(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   128(<rp=%rdi),>mulrax=%rax
movq   128(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#2)
# asm 2: mulq  104(<qp=%rsi)
mulq  104(%rsi)

# qhasm:   carry? c4 += mulrax
# asm 1: add  <mulrax=int64#7,<c4=int64#13
# asm 2: add  <mulrax=%rax,<c4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = *(uint64 *)(rp + 128)
# asm 1: movq   128(<rp=int64#1),>mulrax=int64#3
# asm 2: movq   128(<rp=%rdi),>mulrax=%rdx
movq   128(%rdi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#2)
# asm 2: mulq  112(<qp=%rsi)
mulq  112(%rsi)

# qhasm:   carry? c0 += mulrax
# asm 1: add  <mulrax=int64#7,<c0=int64#4
# asm 2: add  <mulrax=%rax,<c0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = *(uint64 *)(rp + 136)
# asm 1: movq   136(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   136(<rp=%rdi),>mulrax=%rax
movq   136(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#2)
# asm 2: mulq  80(<qp=%rsi)
mulq  80(%rsi)

# qhasm:   carry? c2 += mulrax
# asm 1: add  <mulrax=int64#7,<c2=int64#9
# asm 2: add  <mulrax=%rax,<c2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = *(uint64 *)(rp + 136)
# asm 1: movq   136(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   136(<rp=%rdi),>mulrax=%rax
movq   136(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#2)
# asm 2: mulq  88(<qp=%rsi)
mulq  88(%rsi)

# qhasm:   carry? c3 += mulrax
# asm 1: add  <mulrax=int64#7,<c3=int64#11
# asm 2: add  <mulrax=%rax,<c3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulrax = *(uint64 *)(rp + 136)
# asm 1: movq   136(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   136(<rp=%rdi),>mulrax=%rax
movq   136(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#2)
# asm 2: mulq  96(<qp=%rsi)
mulq  96(%rsi)

# qhasm:   carry? c4 += mulrax
# asm 1: add  <mulrax=int64#7,<c4=int64#13
# asm 2: add  <mulrax=%rax,<c4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = *(uint64 *)(rp + 136)
# asm 1: movq   136(<rp=int64#1),>mulrax=int64#3
# asm 2: movq   136(<rp=%rdi),>mulrax=%rdx
movq   136(%rdi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#2)
# asm 2: mulq  104(<qp=%rsi)
mulq  104(%rsi)

# qhasm:   carry? c0 += mulrax
# asm 1: add  <mulrax=int64#7,<c0=int64#4
# asm 2: add  <mulrax=%rax,<c0=%rcx
add  %rax,%rcx

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#5
# asm 2: adc <mulrdx=%rdx,<mulr01=%r8
adc %rdx,%r8

# qhasm:   mulrax = *(uint64 *)(rp + 136)
# asm 1: movq   136(<rp=int64#1),>mulrax=int64#3
# asm 2: movq   136(<rp=%rdi),>mulrax=%rdx
movq   136(%rdi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#2)
# asm 2: mulq  112(<qp=%rsi)
mulq  112(%rsi)

# qhasm:   carry? c1 += mulrax
# asm 1: add  <mulrax=int64#7,<c1=int64#6
# asm 2: add  <mulrax=%rax,<c1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = *(uint64 *)(rp + 144)
# asm 1: movq   144(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   144(<rp=%rdi),>mulrax=%rax
movq   144(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#2)
# asm 2: mulq  80(<qp=%rsi)
mulq  80(%rsi)

# qhasm:   carry? c3 += mulrax
# asm 1: add  <mulrax=int64#7,<c3=int64#11
# asm 2: add  <mulrax=%rax,<c3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulrax = *(uint64 *)(rp + 144)
# asm 1: movq   144(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   144(<rp=%rdi),>mulrax=%rax
movq   144(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#2)
# asm 2: mulq  88(<qp=%rsi)
mulq  88(%rsi)

# qhasm:   carry? c4 += mulrax
# asm 1: add  <mulrax=int64#7,<c4=int64#13
# asm 2: add  <mulrax=%rax,<c4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#2)
# asm 2: mulq  104(<qp=%rsi)
mulq  104(%rsi)

# qhasm:   carry? c1 += mulrax
# asm 1: add  <mulrax=int64#7,<c1=int64#6
# asm 2: add  <mulrax=%rax,<c1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#2)
# asm 2: mulq  112(<qp=%rsi)
mulq  112(%rsi)

# qhasm:   carry? c2 += mulrax
# asm 1: add  <mulrax=int64#7,<c2=int64#9
# asm 2: add  <mulrax=%rax,<c2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = *(uint64 *)(rp + 152)
# asm 1: movq   152(<rp=int64#1),>mulrax=int64#7
# asm 2: movq   152(<rp=%rdi),>mulrax=%rax
movq   152(%rdi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#2)
# asm 2: mulq  80(<qp=%rsi)
mulq  80(%rsi)

# qhasm:   carry? c4 += mulrax
# asm 1: add  <mulrax=int64#7,<c4=int64#13
# asm 2: add  <mulrax=%rax,<c4=%r15
add  %rax,%r15

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#14
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbx
adc %rdx,%rbx

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#2)
# asm 2: mulq  96(<qp=%rsi)
mulq  96(%rsi)

# qhasm:   carry? c1 += mulrax
# asm 1: add  <mulrax=int64#7,<c1=int64#6
# asm 2: add  <mulrax=%rax,<c1=%r9
add  %rax,%r9

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#8
# asm 2: adc <mulrdx=%rdx,<mulr11=%r10
adc %rdx,%r10

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#2)
# asm 2: mulq  104(<qp=%rsi)
mulq  104(%rsi)

# qhasm:   carry? c2 += mulrax
# asm 1: add  <mulrax=int64#7,<c2=int64#9
# asm 2: add  <mulrax=%rax,<c2=%r11
add  %rax,%r11

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#10
# asm 2: adc <mulrdx=%rdx,<mulr21=%r12
adc %rdx,%r12

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#2)
# asm 2: mulq  112(<qp=%rsi)
mulq  112(%rsi)

# qhasm:   carry? c3 += mulrax
# asm 1: add  <mulrax=int64#7,<c3=int64#11
# asm 2: add  <mulrax=%rax,<c3=%r13
add  %rax,%r13

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#12
# asm 2: adc <mulrdx=%rdx,<mulr31=%r14
adc %rdx,%r14

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#2
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rsi
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rsi

# qhasm:   mulr01 = (mulr01.c0) << 13
# asm 1: shld $13,<c0=int64#4,<mulr01=int64#5
# asm 2: shld $13,<c0=%rcx,<mulr01=%r8
shld $13,%rcx,%r8

# qhasm:   c0 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c0=int64#4
# asm 2: and  <mulredmask=%rsi,<c0=%rcx
and  %rsi,%rcx

# qhasm:   mulr11 = (mulr11.c1) << 13
# asm 1: shld $13,<c1=int64#6,<mulr11=int64#8
# asm 2: shld $13,<c1=%r9,<mulr11=%r10
shld $13,%r9,%r10

# qhasm:   c1 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c1=int64#6
# asm 2: and  <mulredmask=%rsi,<c1=%r9
and  %rsi,%r9

# qhasm:   c1 += mulr01
# asm 1: add  <mulr01=int64#5,<c1=int64#6
# asm 2: add  <mulr01=%r8,<c1=%r9
add  %r8,%r9

# qhasm:   mulr21 = (mulr21.c2) << 13
# asm 1: shld $13,<c2=int64#9,<mulr21=int64#10
# asm 2: shld $13,<c2=%r11,<mulr21=%r12
shld $13,%r11,%r12

# qhasm:   c2 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c2=int64#9
# asm 2: and  <mulredmask=%rsi,<c2=%r11
and  %rsi,%r11

# qhasm:   c2 += mulr11
# asm 1: add  <mulr11=int64#8,<c2=int64#9
# asm 2: add  <mulr11=%r10,<c2=%r11
add  %r10,%r11

# qhasm:   mulr31 = (mulr31.c3) << 13
# asm 1: shld $13,<c3=int64#11,<mulr31=int64#12
# asm 2: shld $13,<c3=%r13,<mulr31=%r14
shld $13,%r13,%r14

# qhasm:   c3 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c3=int64#11
# asm 2: and  <mulredmask=%rsi,<c3=%r13
and  %rsi,%r13

# qhasm:   c3 += mulr21
# asm 1: add  <mulr21=int64#10,<c3=int64#11
# asm 2: add  <mulr21=%r12,<c3=%r13
add  %r12,%r13

# qhasm:   mulr41 = (mulr41.c4) << 13
# asm 1: shld $13,<c4=int64#13,<mulr41=int64#14
# asm 2: shld $13,<c4=%r15,<mulr41=%rbx
shld $13,%r15,%rbx

# qhasm:   c4 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c4=int64#13
# asm 2: and  <mulredmask=%rsi,<c4=%r15
and  %rsi,%r15

# qhasm:   c4 += mulr31
# asm 1: add  <mulr31=int64#12,<c4=int64#13
# asm 2: add  <mulr31=%r14,<c4=%r15
add  %r14,%r15

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#14,>mulr41=int64#3
# asm 2: imulq  $19,<mulr41=%rbx,>mulr41=%rdx
imulq  $19,%rbx,%rdx

# qhasm:   c0 += mulr41
# asm 1: add  <mulr41=int64#3,<c0=int64#4
# asm 2: add  <mulr41=%rdx,<c0=%rcx
add  %rdx,%rcx

# qhasm:   mult = c0
# asm 1: mov  <c0=int64#4,>mult=int64#3
# asm 2: mov  <c0=%rcx,>mult=%rdx
mov  %rcx,%rdx

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#3
# asm 2: shr  $51,<mult=%rdx
shr  $51,%rdx

# qhasm:   mult += c1
# asm 1: add  <c1=int64#6,<mult=int64#3
# asm 2: add  <c1=%r9,<mult=%rdx
add  %r9,%rdx

# qhasm:   c1 = mult
# asm 1: mov  <mult=int64#3,>c1=int64#5
# asm 2: mov  <mult=%rdx,>c1=%r8
mov  %rdx,%r8

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#3
# asm 2: shr  $51,<mult=%rdx
shr  $51,%rdx

# qhasm:   c0 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c0=int64#4
# asm 2: and  <mulredmask=%rsi,<c0=%rcx
and  %rsi,%rcx

# qhasm:   mult += c2
# asm 1: add  <c2=int64#9,<mult=int64#3
# asm 2: add  <c2=%r11,<mult=%rdx
add  %r11,%rdx

# qhasm:   c2 = mult
# asm 1: mov  <mult=int64#3,>c2=int64#6
# asm 2: mov  <mult=%rdx,>c2=%r9
mov  %rdx,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#3
# asm 2: shr  $51,<mult=%rdx
shr  $51,%rdx

# qhasm:   c1 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c1=int64#5
# asm 2: and  <mulredmask=%rsi,<c1=%r8
and  %rsi,%r8

# qhasm:   mult += c3
# asm 1: add  <c3=int64#11,<mult=int64#3
# asm 2: add  <c3=%r13,<mult=%rdx
add  %r13,%rdx

# qhasm:   c3 = mult
# asm 1: mov  <mult=int64#3,>c3=int64#7
# asm 2: mov  <mult=%rdx,>c3=%rax
mov  %rdx,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#3
# asm 2: shr  $51,<mult=%rdx
shr  $51,%rdx

# qhasm:   c2 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c2=int64#6
# asm 2: and  <mulredmask=%rsi,<c2=%r9
and  %rsi,%r9

# qhasm:   mult += c4
# asm 1: add  <c4=int64#13,<mult=int64#3
# asm 2: add  <c4=%r15,<mult=%rdx
add  %r15,%rdx

# qhasm:   c4 = mult
# asm 1: mov  <mult=int64#3,>c4=int64#8
# asm 2: mov  <mult=%rdx,>c4=%r10
mov  %rdx,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#3
# asm 2: shr  $51,<mult=%rdx
shr  $51,%rdx

# qhasm:   c3 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c3=int64#7
# asm 2: and  <mulredmask=%rsi,<c3=%rax
and  %rsi,%rax

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#3,>mult=int64#3
# asm 2: imulq  $19,<mult=%rdx,>mult=%rdx
imulq  $19,%rdx,%rdx

# qhasm:   c0 += mult
# asm 1: add  <mult=int64#3,<c0=int64#4
# asm 2: add  <mult=%rdx,<c0=%rcx
add  %rdx,%rcx

# qhasm:   c4 &= mulredmask
# asm 1: and  <mulredmask=int64#2,<c4=int64#8
# asm 2: and  <mulredmask=%rsi,<c4=%r10
and  %rsi,%r10

# qhasm: c0_stack = c0
# asm 1: movq <c0=int64#4,>c0_stack=stack64#18
# asm 2: movq <c0=%rcx,>c0_stack=136(%rsp)
movq %rcx,136(%rsp)

# qhasm: f0 = *(uint64 *)(rp + 80)
# asm 1: movq   80(<rp=int64#1),>f0=int64#2
# asm 2: movq   80(<rp=%rdi),>f0=%rsi
movq   80(%rdi),%rsi

# qhasm: f1 = *(uint64 *)(rp + 88)
# asm 1: movq   88(<rp=int64#1),>f1=int64#3
# asm 2: movq   88(<rp=%rdi),>f1=%rdx
movq   88(%rdi),%rdx

# qhasm: f2 = *(uint64 *)(rp + 96)
# asm 1: movq   96(<rp=int64#1),>f2=int64#4
# asm 2: movq   96(<rp=%rdi),>f2=%rcx
movq   96(%rdi),%rcx

# qhasm: f3 = *(uint64 *)(rp + 104)
# asm 1: movq   104(<rp=int64#1),>f3=int64#9
# asm 2: movq   104(<rp=%rdi),>f3=%r11
movq   104(%rdi),%r11

# qhasm: f4 = *(uint64 *)(rp + 112)
# asm 1: movq   112(<rp=int64#1),>f4=int64#10
# asm 2: movq   112(<rp=%rdi),>f4=%r12
movq   112(%rdi),%r12

# qhasm: f0 += f0
# asm 1: add  <f0=int64#2,<f0=int64#2
# asm 2: add  <f0=%rsi,<f0=%rsi
add  %rsi,%rsi

# qhasm: f1 += f1
# asm 1: add  <f1=int64#3,<f1=int64#3
# asm 2: add  <f1=%rdx,<f1=%rdx
add  %rdx,%rdx

# qhasm: f2 += f2
# asm 1: add  <f2=int64#4,<f2=int64#4
# asm 2: add  <f2=%rcx,<f2=%rcx
add  %rcx,%rcx

# qhasm: f3 += f3
# asm 1: add  <f3=int64#9,<f3=int64#9
# asm 2: add  <f3=%r11,<f3=%r11
add  %r11,%r11

# qhasm: f4 += f4
# asm 1: add  <f4=int64#10,<f4=int64#10
# asm 2: add  <f4=%r12,<f4=%r12
add  %r12,%r12

# qhasm: g0 = f0
# asm 1: mov  <f0=int64#2,>g0=int64#11
# asm 2: mov  <f0=%rsi,>g0=%r13
mov  %rsi,%r13

# qhasm: g1 = f1
# asm 1: mov  <f1=int64#3,>g1=int64#12
# asm 2: mov  <f1=%rdx,>g1=%r14
mov  %rdx,%r14

# qhasm: g2 = f2
# asm 1: mov  <f2=int64#4,>g2=int64#13
# asm 2: mov  <f2=%rcx,>g2=%r15
mov  %rcx,%r15

# qhasm: g3 = f3
# asm 1: mov  <f3=int64#9,>g3=int64#14
# asm 2: mov  <f3=%r11,>g3=%rbx
mov  %r11,%rbx

# qhasm: g4 = f4
# asm 1: mov  <f4=int64#10,>g4=int64#15
# asm 2: mov  <f4=%r12,>g4=%rbp
mov  %r12,%rbp

# qhasm: f0 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P0
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<f0=int64#2
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<f0=%rsi
add  crypto_sign_ed25519_amd64_51_30k_batch_2P0(%rip),%rsi

# qhasm: f1 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f1=int64#3
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f1=%rdx
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%rdx

# qhasm: f2 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f2=int64#4
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f2=%rcx
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%rcx

# qhasm: f3 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f3=int64#9
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f3=%r11
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r11

# qhasm: f4 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f4=int64#10
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f4=%r12
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r12

# qhasm: g0 += c0_stack
# asm 1: addq <c0_stack=stack64#18,<g0=int64#11
# asm 2: addq <c0_stack=136(%rsp),<g0=%r13
addq 136(%rsp),%r13

# qhasm: g1 += c1
# asm 1: add  <c1=int64#5,<g1=int64#12
# asm 2: add  <c1=%r8,<g1=%r14
add  %r8,%r14

# qhasm: g2 += c2
# asm 1: add  <c2=int64#6,<g2=int64#13
# asm 2: add  <c2=%r9,<g2=%r15
add  %r9,%r15

# qhasm: g3 += c3
# asm 1: add  <c3=int64#7,<g3=int64#14
# asm 2: add  <c3=%rax,<g3=%rbx
add  %rax,%rbx

# qhasm: g4 += c4
# asm 1: add  <c4=int64#8,<g4=int64#15
# asm 2: add  <c4=%r10,<g4=%rbp
add  %r10,%rbp

# qhasm: f0 -= c0_stack
# asm 1: subq <c0_stack=stack64#18,<f0=int64#2
# asm 2: subq <c0_stack=136(%rsp),<f0=%rsi
subq 136(%rsp),%rsi

# qhasm: f1 -= c1
# asm 1: sub  <c1=int64#5,<f1=int64#3
# asm 2: sub  <c1=%r8,<f1=%rdx
sub  %r8,%rdx

# qhasm: f2 -= c2
# asm 1: sub  <c2=int64#6,<f2=int64#4
# asm 2: sub  <c2=%r9,<f2=%rcx
sub  %r9,%rcx

# qhasm: f3 -= c3
# asm 1: sub  <c3=int64#7,<f3=int64#9
# asm 2: sub  <c3=%rax,<f3=%r11
sub  %rax,%r11

# qhasm: f4 -= c4
# asm 1: sub  <c4=int64#8,<f4=int64#10
# asm 2: sub  <c4=%r10,<f4=%r12
sub  %r10,%r12

# qhasm: g0_stack = g0
# asm 1: movq <g0=int64#11,>g0_stack=stack64#18
# asm 2: movq <g0=%r13,>g0_stack=136(%rsp)
movq %r13,136(%rsp)

# qhasm: g1_stack = g1
# asm 1: movq <g1=int64#12,>g1_stack=stack64#19
# asm 2: movq <g1=%r14,>g1_stack=144(%rsp)
movq %r14,144(%rsp)

# qhasm: g2_stack = g2
# asm 1: movq <g2=int64#13,>g2_stack=stack64#20
# asm 2: movq <g2=%r15,>g2_stack=152(%rsp)
movq %r15,152(%rsp)

# qhasm: g3_stack = g3
# asm 1: movq <g3=int64#14,>g3_stack=stack64#21
# asm 2: movq <g3=%rbx,>g3_stack=160(%rsp)
movq %rbx,160(%rsp)

# qhasm: g4_stack = g4
# asm 1: movq <g4=int64#15,>g4_stack=stack64#22
# asm 2: movq <g4=%rbp,>g4_stack=168(%rsp)
movq %rbp,168(%rsp)

# qhasm: f0_stack = f0
# asm 1: movq <f0=int64#2,>f0_stack=stack64#23
# asm 2: movq <f0=%rsi,>f0_stack=176(%rsp)
movq %rsi,176(%rsp)

# qhasm: f1_stack = f1
# asm 1: movq <f1=int64#3,>f1_stack=stack64#24
# asm 2: movq <f1=%rdx,>f1_stack=184(%rsp)
movq %rdx,184(%rsp)

# qhasm: f2_stack = f2
# asm 1: movq <f2=int64#4,>f2_stack=stack64#25
# asm 2: movq <f2=%rcx,>f2_stack=192(%rsp)
movq %rcx,192(%rsp)

# qhasm: f3_stack = f3
# asm 1: movq <f3=int64#9,>f3_stack=stack64#26
# asm 2: movq <f3=%r11,>f3_stack=200(%rsp)
movq %r11,200(%rsp)

# qhasm: f4_stack = f4
# asm 1: movq <f4=int64#10,>f4_stack=stack64#27
# asm 2: movq <f4=%r12,>f4_stack=208(%rsp)
movq %r12,208(%rsp)

# qhasm:   mulrax = e3_stack
# asm 1: movq <e3_stack=stack64#16,>mulrax=int64#2
# asm 2: movq <e3_stack=120(%rsp),>mulrax=%rsi
movq 120(%rsp),%rsi

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#2,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rsi,>mulrax=%rax
imulq  $19,%rsi,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#28
# asm 2: movq <mulrax=%rax,>mulx319_stack=216(%rsp)
movq %rax,216(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   rx0 = mulrax
# asm 1: mov  <mulrax=int64#7,>rx0=int64#2
# asm 2: mov  <mulrax=%rax,>rx0=%rsi
mov  %rax,%rsi

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#4
# asm 2: mov  <mulrdx=%rdx,>mulr01=%rcx
mov  %rdx,%rcx

# qhasm:   mulrax = e4_stack
# asm 1: movq <e4_stack=stack64#17,>mulrax=int64#3
# asm 2: movq <e4_stack=128(%rsp),>mulrax=%rdx
movq 128(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#29
# asm 2: movq <mulrax=%rax,>mulx419_stack=224(%rsp)
movq %rax,224(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   carry? rx0 += mulrax
# asm 1: add  <mulrax=int64#7,<rx0=int64#2
# asm 2: add  <mulrax=%rax,<rx0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rx0 += mulrax
# asm 1: add  <mulrax=int64#7,<rx0=int64#2
# asm 2: add  <mulrax=%rax,<rx0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   rx1 = mulrax
# asm 1: mov  <mulrax=int64#7,>rx1=int64#5
# asm 2: mov  <mulrax=%rax,>rx1=%r8
mov  %rax,%r8

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#6
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r9
mov  %rdx,%r9

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   rx2 = mulrax
# asm 1: mov  <mulrax=int64#7,>rx2=int64#8
# asm 2: mov  <mulrax=%rax,>rx2=%r10
mov  %rax,%r10

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#9
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r11
mov  %rdx,%r11

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   rx3 = mulrax
# asm 1: mov  <mulrax=int64#7,>rx3=int64#10
# asm 2: mov  <mulrax=%rax,>rx3=%r12
mov  %rax,%r12

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#11
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r13
mov  %rdx,%r13

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   rx4 = mulrax
# asm 1: mov  <mulrax=int64#7,>rx4=int64#12
# asm 2: mov  <mulrax=%rax,>rx4=%r14
mov  %rax,%r14

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#13
# asm 2: mov  <mulrdx=%rdx,>mulr41=%r15
mov  %rdx,%r15

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rx1 += mulrax
# asm 1: add  <mulrax=int64#7,<rx1=int64#5
# asm 2: add  <mulrax=%rax,<rx1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   carry? rx2 += mulrax
# asm 1: add  <mulrax=int64#7,<rx2=int64#8
# asm 2: add  <mulrax=%rax,<rx2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   carry? rx3 += mulrax
# asm 1: add  <mulrax=int64#7,<rx3=int64#10
# asm 2: add  <mulrax=%rax,<rx3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   carry? rx4 += mulrax
# asm 1: add  <mulrax=int64#7,<rx4=int64#12
# asm 2: add  <mulrax=%rax,<rx4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#3
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rdx
movq 104(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   carry? rx0 += mulrax
# asm 1: add  <mulrax=int64#7,<rx0=int64#2
# asm 2: add  <mulrax=%rax,<rx0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rx2 += mulrax
# asm 1: add  <mulrax=int64#7,<rx2=int64#8
# asm 2: add  <mulrax=%rax,<rx2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   carry? rx3 += mulrax
# asm 1: add  <mulrax=int64#7,<rx3=int64#10
# asm 2: add  <mulrax=%rax,<rx3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   carry? rx4 += mulrax
# asm 1: add  <mulrax=int64#7,<rx4=int64#12
# asm 2: add  <mulrax=%rax,<rx4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#3
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rdx
movq 112(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   carry? rx0 += mulrax
# asm 1: add  <mulrax=int64#7,<rx0=int64#2
# asm 2: add  <mulrax=%rax,<rx0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#3
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rdx
movq 112(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   carry? rx1 += mulrax
# asm 1: add  <mulrax=int64#7,<rx1=int64#5
# asm 2: add  <mulrax=%rax,<rx1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = e3_stack
# asm 1: movq <e3_stack=stack64#16,>mulrax=int64#7
# asm 2: movq <e3_stack=120(%rsp),>mulrax=%rax
movq 120(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rx3 += mulrax
# asm 1: add  <mulrax=int64#7,<rx3=int64#10
# asm 2: add  <mulrax=%rax,<rx3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = e3_stack
# asm 1: movq <e3_stack=stack64#16,>mulrax=int64#7
# asm 2: movq <e3_stack=120(%rsp),>mulrax=%rax
movq 120(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   carry? rx4 += mulrax
# asm 1: add  <mulrax=int64#7,<rx4=int64#12
# asm 2: add  <mulrax=%rax,<rx4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#28,>mulrax=int64#7
# asm 2: movq <mulx319_stack=216(%rsp),>mulrax=%rax
movq 216(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   carry? rx1 += mulrax
# asm 1: add  <mulrax=int64#7,<rx1=int64#5
# asm 2: add  <mulrax=%rax,<rx1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#28,>mulrax=int64#7
# asm 2: movq <mulx319_stack=216(%rsp),>mulrax=%rax
movq 216(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   carry? rx2 += mulrax
# asm 1: add  <mulrax=int64#7,<rx2=int64#8
# asm 2: add  <mulrax=%rax,<rx2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = e4_stack
# asm 1: movq <e4_stack=stack64#17,>mulrax=int64#7
# asm 2: movq <e4_stack=128(%rsp),>mulrax=%rax
movq 128(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rx4 += mulrax
# asm 1: add  <mulrax=int64#7,<rx4=int64#12
# asm 2: add  <mulrax=%rax,<rx4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#29,>mulrax=int64#7
# asm 2: movq <mulx419_stack=224(%rsp),>mulrax=%rax
movq 224(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   carry? rx1 += mulrax
# asm 1: add  <mulrax=int64#7,<rx1=int64#5
# asm 2: add  <mulrax=%rax,<rx1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#29,>mulrax=int64#7
# asm 2: movq <mulx419_stack=224(%rsp),>mulrax=%rax
movq 224(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   carry? rx2 += mulrax
# asm 1: add  <mulrax=int64#7,<rx2=int64#8
# asm 2: add  <mulrax=%rax,<rx2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#29,>mulrax=int64#7
# asm 2: movq <mulx419_stack=224(%rsp),>mulrax=%rax
movq 224(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   carry? rx3 += mulrax
# asm 1: add  <mulrax=int64#7,<rx3=int64#10
# asm 2: add  <mulrax=%rax,<rx3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#3
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rdx
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rdx

# qhasm:   mulr01 = (mulr01.rx0) << 13
# asm 1: shld $13,<rx0=int64#2,<mulr01=int64#4
# asm 2: shld $13,<rx0=%rsi,<mulr01=%rcx
shld $13,%rsi,%rcx

# qhasm:   rx0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx0=int64#2
# asm 2: and  <mulredmask=%rdx,<rx0=%rsi
and  %rdx,%rsi

# qhasm:   mulr11 = (mulr11.rx1) << 13
# asm 1: shld $13,<rx1=int64#5,<mulr11=int64#6
# asm 2: shld $13,<rx1=%r8,<mulr11=%r9
shld $13,%r8,%r9

# qhasm:   rx1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx1=int64#5
# asm 2: and  <mulredmask=%rdx,<rx1=%r8
and  %rdx,%r8

# qhasm:   rx1 += mulr01
# asm 1: add  <mulr01=int64#4,<rx1=int64#5
# asm 2: add  <mulr01=%rcx,<rx1=%r8
add  %rcx,%r8

# qhasm:   mulr21 = (mulr21.rx2) << 13
# asm 1: shld $13,<rx2=int64#8,<mulr21=int64#9
# asm 2: shld $13,<rx2=%r10,<mulr21=%r11
shld $13,%r10,%r11

# qhasm:   rx2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx2=int64#8
# asm 2: and  <mulredmask=%rdx,<rx2=%r10
and  %rdx,%r10

# qhasm:   rx2 += mulr11
# asm 1: add  <mulr11=int64#6,<rx2=int64#8
# asm 2: add  <mulr11=%r9,<rx2=%r10
add  %r9,%r10

# qhasm:   mulr31 = (mulr31.rx3) << 13
# asm 1: shld $13,<rx3=int64#10,<mulr31=int64#11
# asm 2: shld $13,<rx3=%r12,<mulr31=%r13
shld $13,%r12,%r13

# qhasm:   rx3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx3=int64#10
# asm 2: and  <mulredmask=%rdx,<rx3=%r12
and  %rdx,%r12

# qhasm:   rx3 += mulr21
# asm 1: add  <mulr21=int64#9,<rx3=int64#10
# asm 2: add  <mulr21=%r11,<rx3=%r12
add  %r11,%r12

# qhasm:   mulr41 = (mulr41.rx4) << 13
# asm 1: shld $13,<rx4=int64#12,<mulr41=int64#13
# asm 2: shld $13,<rx4=%r14,<mulr41=%r15
shld $13,%r14,%r15

# qhasm:   rx4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx4=int64#12
# asm 2: and  <mulredmask=%rdx,<rx4=%r14
and  %rdx,%r14

# qhasm:   rx4 += mulr31
# asm 1: add  <mulr31=int64#11,<rx4=int64#12
# asm 2: add  <mulr31=%r13,<rx4=%r14
add  %r13,%r14

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#13,>mulr41=int64#4
# asm 2: imulq  $19,<mulr41=%r15,>mulr41=%rcx
imulq  $19,%r15,%rcx

# qhasm:   rx0 += mulr41
# asm 1: add  <mulr41=int64#4,<rx0=int64#2
# asm 2: add  <mulr41=%rcx,<rx0=%rsi
add  %rcx,%rsi

# qhasm:   mult = rx0
# asm 1: mov  <rx0=int64#2,>mult=int64#4
# asm 2: mov  <rx0=%rsi,>mult=%rcx
mov  %rsi,%rcx

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   mult += rx1
# asm 1: add  <rx1=int64#5,<mult=int64#4
# asm 2: add  <rx1=%r8,<mult=%rcx
add  %r8,%rcx

# qhasm:   rx1 = mult
# asm 1: mov  <mult=int64#4,>rx1=int64#5
# asm 2: mov  <mult=%rcx,>rx1=%r8
mov  %rcx,%r8

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rx0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx0=int64#2
# asm 2: and  <mulredmask=%rdx,<rx0=%rsi
and  %rdx,%rsi

# qhasm:   mult += rx2
# asm 1: add  <rx2=int64#8,<mult=int64#4
# asm 2: add  <rx2=%r10,<mult=%rcx
add  %r10,%rcx

# qhasm:   rx2 = mult
# asm 1: mov  <mult=int64#4,>rx2=int64#6
# asm 2: mov  <mult=%rcx,>rx2=%r9
mov  %rcx,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rx1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx1=int64#5
# asm 2: and  <mulredmask=%rdx,<rx1=%r8
and  %rdx,%r8

# qhasm:   mult += rx3
# asm 1: add  <rx3=int64#10,<mult=int64#4
# asm 2: add  <rx3=%r12,<mult=%rcx
add  %r12,%rcx

# qhasm:   rx3 = mult
# asm 1: mov  <mult=int64#4,>rx3=int64#7
# asm 2: mov  <mult=%rcx,>rx3=%rax
mov  %rcx,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rx2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx2=int64#6
# asm 2: and  <mulredmask=%rdx,<rx2=%r9
and  %rdx,%r9

# qhasm:   mult += rx4
# asm 1: add  <rx4=int64#12,<mult=int64#4
# asm 2: add  <rx4=%r14,<mult=%rcx
add  %r14,%rcx

# qhasm:   rx4 = mult
# asm 1: mov  <mult=int64#4,>rx4=int64#8
# asm 2: mov  <mult=%rcx,>rx4=%r10
mov  %rcx,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rx3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx3=int64#7
# asm 2: and  <mulredmask=%rdx,<rx3=%rax
and  %rdx,%rax

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#4,>mult=int64#4
# asm 2: imulq  $19,<mult=%rcx,>mult=%rcx
imulq  $19,%rcx,%rcx

# qhasm:   rx0 += mult
# asm 1: add  <mult=int64#4,<rx0=int64#2
# asm 2: add  <mult=%rcx,<rx0=%rsi
add  %rcx,%rsi

# qhasm:   rx4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rx4=int64#8
# asm 2: and  <mulredmask=%rdx,<rx4=%r10
and  %rdx,%r10

# qhasm: *(uint64 *)(rp + 0) = rx0
# asm 1: movq   <rx0=int64#2,0(<rp=int64#1)
# asm 2: movq   <rx0=%rsi,0(<rp=%rdi)
movq   %rsi,0(%rdi)

# qhasm: *(uint64 *)(rp + 8) = rx1
# asm 1: movq   <rx1=int64#5,8(<rp=int64#1)
# asm 2: movq   <rx1=%r8,8(<rp=%rdi)
movq   %r8,8(%rdi)

# qhasm: *(uint64 *)(rp + 16) = rx2
# asm 1: movq   <rx2=int64#6,16(<rp=int64#1)
# asm 2: movq   <rx2=%r9,16(<rp=%rdi)
movq   %r9,16(%rdi)

# qhasm: *(uint64 *)(rp + 24) = rx3
# asm 1: movq   <rx3=int64#7,24(<rp=int64#1)
# asm 2: movq   <rx3=%rax,24(<rp=%rdi)
movq   %rax,24(%rdi)

# qhasm: *(uint64 *)(rp + 32) = rx4
# asm 1: movq   <rx4=int64#8,32(<rp=int64#1)
# asm 2: movq   <rx4=%r10,32(<rp=%rdi)
movq   %r10,32(%rdi)

# qhasm:   mulrax = h3_stack
# asm 1: movq <h3_stack=stack64#11,>mulrax=int64#2
# asm 2: movq <h3_stack=80(%rsp),>mulrax=%rsi
movq 80(%rsp),%rsi

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#2,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rsi,>mulrax=%rax
imulq  $19,%rsi,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#28
# asm 2: movq <mulrax=%rax,>mulx319_stack=216(%rsp)
movq %rax,216(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * g2_stack
# asm 1: mulq  <g2_stack=stack64#20
# asm 2: mulq  <g2_stack=152(%rsp)
mulq  152(%rsp)

# qhasm:   ry0 = mulrax
# asm 1: mov  <mulrax=int64#7,>ry0=int64#2
# asm 2: mov  <mulrax=%rax,>ry0=%rsi
mov  %rax,%rsi

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#4
# asm 2: mov  <mulrdx=%rdx,>mulr01=%rcx
mov  %rdx,%rcx

# qhasm:   mulrax = h4_stack
# asm 1: movq <h4_stack=stack64#12,>mulrax=int64#3
# asm 2: movq <h4_stack=88(%rsp),>mulrax=%rdx
movq 88(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#29
# asm 2: movq <mulrax=%rax,>mulx419_stack=224(%rsp)
movq %rax,224(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * g1_stack
# asm 1: mulq  <g1_stack=stack64#19
# asm 2: mulq  <g1_stack=144(%rsp)
mulq  144(%rsp)

# qhasm:   carry? ry0 += mulrax
# asm 1: add  <mulrax=int64#7,<ry0=int64#2
# asm 2: add  <mulrax=%rax,<ry0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = h0_stack
# asm 1: movq <h0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <h0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g0_stack
# asm 1: mulq  <g0_stack=stack64#18
# asm 2: mulq  <g0_stack=136(%rsp)
mulq  136(%rsp)

# qhasm:   carry? ry0 += mulrax
# asm 1: add  <mulrax=int64#7,<ry0=int64#2
# asm 2: add  <mulrax=%rax,<ry0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = h0_stack
# asm 1: movq <h0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <h0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g1_stack
# asm 1: mulq  <g1_stack=stack64#19
# asm 2: mulq  <g1_stack=144(%rsp)
mulq  144(%rsp)

# qhasm:   ry1 = mulrax
# asm 1: mov  <mulrax=int64#7,>ry1=int64#5
# asm 2: mov  <mulrax=%rax,>ry1=%r8
mov  %rax,%r8

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#6
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r9
mov  %rdx,%r9

# qhasm:   mulrax = h0_stack
# asm 1: movq <h0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <h0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g2_stack
# asm 1: mulq  <g2_stack=stack64#20
# asm 2: mulq  <g2_stack=152(%rsp)
mulq  152(%rsp)

# qhasm:   ry2 = mulrax
# asm 1: mov  <mulrax=int64#7,>ry2=int64#8
# asm 2: mov  <mulrax=%rax,>ry2=%r10
mov  %rax,%r10

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#9
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r11
mov  %rdx,%r11

# qhasm:   mulrax = h0_stack
# asm 1: movq <h0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <h0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g3_stack
# asm 1: mulq  <g3_stack=stack64#21
# asm 2: mulq  <g3_stack=160(%rsp)
mulq  160(%rsp)

# qhasm:   ry3 = mulrax
# asm 1: mov  <mulrax=int64#7,>ry3=int64#10
# asm 2: mov  <mulrax=%rax,>ry3=%r12
mov  %rax,%r12

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#11
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r13
mov  %rdx,%r13

# qhasm:   mulrax = h0_stack
# asm 1: movq <h0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <h0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g4_stack
# asm 1: mulq  <g4_stack=stack64#22
# asm 2: mulq  <g4_stack=168(%rsp)
mulq  168(%rsp)

# qhasm:   ry4 = mulrax
# asm 1: mov  <mulrax=int64#7,>ry4=int64#12
# asm 2: mov  <mulrax=%rax,>ry4=%r14
mov  %rax,%r14

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#13
# asm 2: mov  <mulrdx=%rdx,>mulr41=%r15
mov  %rdx,%r15

# qhasm:   mulrax = h1_stack
# asm 1: movq <h1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <h1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g0_stack
# asm 1: mulq  <g0_stack=stack64#18
# asm 2: mulq  <g0_stack=136(%rsp)
mulq  136(%rsp)

# qhasm:   carry? ry1 += mulrax
# asm 1: add  <mulrax=int64#7,<ry1=int64#5
# asm 2: add  <mulrax=%rax,<ry1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = h1_stack
# asm 1: movq <h1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <h1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g1_stack
# asm 1: mulq  <g1_stack=stack64#19
# asm 2: mulq  <g1_stack=144(%rsp)
mulq  144(%rsp)

# qhasm:   carry? ry2 += mulrax
# asm 1: add  <mulrax=int64#7,<ry2=int64#8
# asm 2: add  <mulrax=%rax,<ry2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = h1_stack
# asm 1: movq <h1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <h1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g2_stack
# asm 1: mulq  <g2_stack=stack64#20
# asm 2: mulq  <g2_stack=152(%rsp)
mulq  152(%rsp)

# qhasm:   carry? ry3 += mulrax
# asm 1: add  <mulrax=int64#7,<ry3=int64#10
# asm 2: add  <mulrax=%rax,<ry3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = h1_stack
# asm 1: movq <h1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <h1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g3_stack
# asm 1: mulq  <g3_stack=stack64#21
# asm 2: mulq  <g3_stack=160(%rsp)
mulq  160(%rsp)

# qhasm:   carry? ry4 += mulrax
# asm 1: add  <mulrax=int64#7,<ry4=int64#12
# asm 2: add  <mulrax=%rax,<ry4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = h1_stack
# asm 1: movq <h1_stack=stack64#9,>mulrax=int64#3
# asm 2: movq <h1_stack=64(%rsp),>mulrax=%rdx
movq 64(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g4_stack
# asm 1: mulq  <g4_stack=stack64#22
# asm 2: mulq  <g4_stack=168(%rsp)
mulq  168(%rsp)

# qhasm:   carry? ry0 += mulrax
# asm 1: add  <mulrax=int64#7,<ry0=int64#2
# asm 2: add  <mulrax=%rax,<ry0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = h2_stack
# asm 1: movq <h2_stack=stack64#10,>mulrax=int64#7
# asm 2: movq <h2_stack=72(%rsp),>mulrax=%rax
movq 72(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g0_stack
# asm 1: mulq  <g0_stack=stack64#18
# asm 2: mulq  <g0_stack=136(%rsp)
mulq  136(%rsp)

# qhasm:   carry? ry2 += mulrax
# asm 1: add  <mulrax=int64#7,<ry2=int64#8
# asm 2: add  <mulrax=%rax,<ry2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = h2_stack
# asm 1: movq <h2_stack=stack64#10,>mulrax=int64#7
# asm 2: movq <h2_stack=72(%rsp),>mulrax=%rax
movq 72(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g1_stack
# asm 1: mulq  <g1_stack=stack64#19
# asm 2: mulq  <g1_stack=144(%rsp)
mulq  144(%rsp)

# qhasm:   carry? ry3 += mulrax
# asm 1: add  <mulrax=int64#7,<ry3=int64#10
# asm 2: add  <mulrax=%rax,<ry3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = h2_stack
# asm 1: movq <h2_stack=stack64#10,>mulrax=int64#7
# asm 2: movq <h2_stack=72(%rsp),>mulrax=%rax
movq 72(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g2_stack
# asm 1: mulq  <g2_stack=stack64#20
# asm 2: mulq  <g2_stack=152(%rsp)
mulq  152(%rsp)

# qhasm:   carry? ry4 += mulrax
# asm 1: add  <mulrax=int64#7,<ry4=int64#12
# asm 2: add  <mulrax=%rax,<ry4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = h2_stack
# asm 1: movq <h2_stack=stack64#10,>mulrax=int64#3
# asm 2: movq <h2_stack=72(%rsp),>mulrax=%rdx
movq 72(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g3_stack
# asm 1: mulq  <g3_stack=stack64#21
# asm 2: mulq  <g3_stack=160(%rsp)
mulq  160(%rsp)

# qhasm:   carry? ry0 += mulrax
# asm 1: add  <mulrax=int64#7,<ry0=int64#2
# asm 2: add  <mulrax=%rax,<ry0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = h2_stack
# asm 1: movq <h2_stack=stack64#10,>mulrax=int64#3
# asm 2: movq <h2_stack=72(%rsp),>mulrax=%rdx
movq 72(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g4_stack
# asm 1: mulq  <g4_stack=stack64#22
# asm 2: mulq  <g4_stack=168(%rsp)
mulq  168(%rsp)

# qhasm:   carry? ry1 += mulrax
# asm 1: add  <mulrax=int64#7,<ry1=int64#5
# asm 2: add  <mulrax=%rax,<ry1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = h3_stack
# asm 1: movq <h3_stack=stack64#11,>mulrax=int64#7
# asm 2: movq <h3_stack=80(%rsp),>mulrax=%rax
movq 80(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g0_stack
# asm 1: mulq  <g0_stack=stack64#18
# asm 2: mulq  <g0_stack=136(%rsp)
mulq  136(%rsp)

# qhasm:   carry? ry3 += mulrax
# asm 1: add  <mulrax=int64#7,<ry3=int64#10
# asm 2: add  <mulrax=%rax,<ry3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = h3_stack
# asm 1: movq <h3_stack=stack64#11,>mulrax=int64#7
# asm 2: movq <h3_stack=80(%rsp),>mulrax=%rax
movq 80(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g1_stack
# asm 1: mulq  <g1_stack=stack64#19
# asm 2: mulq  <g1_stack=144(%rsp)
mulq  144(%rsp)

# qhasm:   carry? ry4 += mulrax
# asm 1: add  <mulrax=int64#7,<ry4=int64#12
# asm 2: add  <mulrax=%rax,<ry4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#28,>mulrax=int64#7
# asm 2: movq <mulx319_stack=216(%rsp),>mulrax=%rax
movq 216(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g3_stack
# asm 1: mulq  <g3_stack=stack64#21
# asm 2: mulq  <g3_stack=160(%rsp)
mulq  160(%rsp)

# qhasm:   carry? ry1 += mulrax
# asm 1: add  <mulrax=int64#7,<ry1=int64#5
# asm 2: add  <mulrax=%rax,<ry1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#28,>mulrax=int64#7
# asm 2: movq <mulx319_stack=216(%rsp),>mulrax=%rax
movq 216(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g4_stack
# asm 1: mulq  <g4_stack=stack64#22
# asm 2: mulq  <g4_stack=168(%rsp)
mulq  168(%rsp)

# qhasm:   carry? ry2 += mulrax
# asm 1: add  <mulrax=int64#7,<ry2=int64#8
# asm 2: add  <mulrax=%rax,<ry2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = h4_stack
# asm 1: movq <h4_stack=stack64#12,>mulrax=int64#7
# asm 2: movq <h4_stack=88(%rsp),>mulrax=%rax
movq 88(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g0_stack
# asm 1: mulq  <g0_stack=stack64#18
# asm 2: mulq  <g0_stack=136(%rsp)
mulq  136(%rsp)

# qhasm:   carry? ry4 += mulrax
# asm 1: add  <mulrax=int64#7,<ry4=int64#12
# asm 2: add  <mulrax=%rax,<ry4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#29,>mulrax=int64#7
# asm 2: movq <mulx419_stack=224(%rsp),>mulrax=%rax
movq 224(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g2_stack
# asm 1: mulq  <g2_stack=stack64#20
# asm 2: mulq  <g2_stack=152(%rsp)
mulq  152(%rsp)

# qhasm:   carry? ry1 += mulrax
# asm 1: add  <mulrax=int64#7,<ry1=int64#5
# asm 2: add  <mulrax=%rax,<ry1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#29,>mulrax=int64#7
# asm 2: movq <mulx419_stack=224(%rsp),>mulrax=%rax
movq 224(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g3_stack
# asm 1: mulq  <g3_stack=stack64#21
# asm 2: mulq  <g3_stack=160(%rsp)
mulq  160(%rsp)

# qhasm:   carry? ry2 += mulrax
# asm 1: add  <mulrax=int64#7,<ry2=int64#8
# asm 2: add  <mulrax=%rax,<ry2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#29,>mulrax=int64#7
# asm 2: movq <mulx419_stack=224(%rsp),>mulrax=%rax
movq 224(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * g4_stack
# asm 1: mulq  <g4_stack=stack64#22
# asm 2: mulq  <g4_stack=168(%rsp)
mulq  168(%rsp)

# qhasm:   carry? ry3 += mulrax
# asm 1: add  <mulrax=int64#7,<ry3=int64#10
# asm 2: add  <mulrax=%rax,<ry3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#3
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rdx
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rdx

# qhasm:   mulr01 = (mulr01.ry0) << 13
# asm 1: shld $13,<ry0=int64#2,<mulr01=int64#4
# asm 2: shld $13,<ry0=%rsi,<mulr01=%rcx
shld $13,%rsi,%rcx

# qhasm:   ry0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry0=int64#2
# asm 2: and  <mulredmask=%rdx,<ry0=%rsi
and  %rdx,%rsi

# qhasm:   mulr11 = (mulr11.ry1) << 13
# asm 1: shld $13,<ry1=int64#5,<mulr11=int64#6
# asm 2: shld $13,<ry1=%r8,<mulr11=%r9
shld $13,%r8,%r9

# qhasm:   ry1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry1=int64#5
# asm 2: and  <mulredmask=%rdx,<ry1=%r8
and  %rdx,%r8

# qhasm:   ry1 += mulr01
# asm 1: add  <mulr01=int64#4,<ry1=int64#5
# asm 2: add  <mulr01=%rcx,<ry1=%r8
add  %rcx,%r8

# qhasm:   mulr21 = (mulr21.ry2) << 13
# asm 1: shld $13,<ry2=int64#8,<mulr21=int64#9
# asm 2: shld $13,<ry2=%r10,<mulr21=%r11
shld $13,%r10,%r11

# qhasm:   ry2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry2=int64#8
# asm 2: and  <mulredmask=%rdx,<ry2=%r10
and  %rdx,%r10

# qhasm:   ry2 += mulr11
# asm 1: add  <mulr11=int64#6,<ry2=int64#8
# asm 2: add  <mulr11=%r9,<ry2=%r10
add  %r9,%r10

# qhasm:   mulr31 = (mulr31.ry3) << 13
# asm 1: shld $13,<ry3=int64#10,<mulr31=int64#11
# asm 2: shld $13,<ry3=%r12,<mulr31=%r13
shld $13,%r12,%r13

# qhasm:   ry3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry3=int64#10
# asm 2: and  <mulredmask=%rdx,<ry3=%r12
and  %rdx,%r12

# qhasm:   ry3 += mulr21
# asm 1: add  <mulr21=int64#9,<ry3=int64#10
# asm 2: add  <mulr21=%r11,<ry3=%r12
add  %r11,%r12

# qhasm:   mulr41 = (mulr41.ry4) << 13
# asm 1: shld $13,<ry4=int64#12,<mulr41=int64#13
# asm 2: shld $13,<ry4=%r14,<mulr41=%r15
shld $13,%r14,%r15

# qhasm:   ry4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry4=int64#12
# asm 2: and  <mulredmask=%rdx,<ry4=%r14
and  %rdx,%r14

# qhasm:   ry4 += mulr31
# asm 1: add  <mulr31=int64#11,<ry4=int64#12
# asm 2: add  <mulr31=%r13,<ry4=%r14
add  %r13,%r14

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#13,>mulr41=int64#4
# asm 2: imulq  $19,<mulr41=%r15,>mulr41=%rcx
imulq  $19,%r15,%rcx

# qhasm:   ry0 += mulr41
# asm 1: add  <mulr41=int64#4,<ry0=int64#2
# asm 2: add  <mulr41=%rcx,<ry0=%rsi
add  %rcx,%rsi

# qhasm:   mult = ry0
# asm 1: mov  <ry0=int64#2,>mult=int64#4
# asm 2: mov  <ry0=%rsi,>mult=%rcx
mov  %rsi,%rcx

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   mult += ry1
# asm 1: add  <ry1=int64#5,<mult=int64#4
# asm 2: add  <ry1=%r8,<mult=%rcx
add  %r8,%rcx

# qhasm:   ry1 = mult
# asm 1: mov  <mult=int64#4,>ry1=int64#5
# asm 2: mov  <mult=%rcx,>ry1=%r8
mov  %rcx,%r8

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   ry0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry0=int64#2
# asm 2: and  <mulredmask=%rdx,<ry0=%rsi
and  %rdx,%rsi

# qhasm:   mult += ry2
# asm 1: add  <ry2=int64#8,<mult=int64#4
# asm 2: add  <ry2=%r10,<mult=%rcx
add  %r10,%rcx

# qhasm:   ry2 = mult
# asm 1: mov  <mult=int64#4,>ry2=int64#6
# asm 2: mov  <mult=%rcx,>ry2=%r9
mov  %rcx,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   ry1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry1=int64#5
# asm 2: and  <mulredmask=%rdx,<ry1=%r8
and  %rdx,%r8

# qhasm:   mult += ry3
# asm 1: add  <ry3=int64#10,<mult=int64#4
# asm 2: add  <ry3=%r12,<mult=%rcx
add  %r12,%rcx

# qhasm:   ry3 = mult
# asm 1: mov  <mult=int64#4,>ry3=int64#7
# asm 2: mov  <mult=%rcx,>ry3=%rax
mov  %rcx,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   ry2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry2=int64#6
# asm 2: and  <mulredmask=%rdx,<ry2=%r9
and  %rdx,%r9

# qhasm:   mult += ry4
# asm 1: add  <ry4=int64#12,<mult=int64#4
# asm 2: add  <ry4=%r14,<mult=%rcx
add  %r14,%rcx

# qhasm:   ry4 = mult
# asm 1: mov  <mult=int64#4,>ry4=int64#8
# asm 2: mov  <mult=%rcx,>ry4=%r10
mov  %rcx,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   ry3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry3=int64#7
# asm 2: and  <mulredmask=%rdx,<ry3=%rax
and  %rdx,%rax

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#4,>mult=int64#4
# asm 2: imulq  $19,<mult=%rcx,>mult=%rcx
imulq  $19,%rcx,%rcx

# qhasm:   ry0 += mult
# asm 1: add  <mult=int64#4,<ry0=int64#2
# asm 2: add  <mult=%rcx,<ry0=%rsi
add  %rcx,%rsi

# qhasm:   ry4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<ry4=int64#8
# asm 2: and  <mulredmask=%rdx,<ry4=%r10
and  %rdx,%r10

# qhasm: *(uint64 *)(rp + 40) = ry0
# asm 1: movq   <ry0=int64#2,40(<rp=int64#1)
# asm 2: movq   <ry0=%rsi,40(<rp=%rdi)
movq   %rsi,40(%rdi)

# qhasm: *(uint64 *)(rp + 48) = ry1
# asm 1: movq   <ry1=int64#5,48(<rp=int64#1)
# asm 2: movq   <ry1=%r8,48(<rp=%rdi)
movq   %r8,48(%rdi)

# qhasm: *(uint64 *)(rp + 56) = ry2
# asm 1: movq   <ry2=int64#6,56(<rp=int64#1)
# asm 2: movq   <ry2=%r9,56(<rp=%rdi)
movq   %r9,56(%rdi)

# qhasm: *(uint64 *)(rp + 64) = ry3
# asm 1: movq   <ry3=int64#7,64(<rp=int64#1)
# asm 2: movq   <ry3=%rax,64(<rp=%rdi)
movq   %rax,64(%rdi)

# qhasm: *(uint64 *)(rp + 72) = ry4
# asm 1: movq   <ry4=int64#8,72(<rp=int64#1)
# asm 2: movq   <ry4=%r10,72(<rp=%rdi)
movq   %r10,72(%rdi)

# qhasm:   mulrax = g3_stack
# asm 1: movq <g3_stack=stack64#21,>mulrax=int64#2
# asm 2: movq <g3_stack=160(%rsp),>mulrax=%rsi
movq 160(%rsp),%rsi

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#2,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rsi,>mulrax=%rax
imulq  $19,%rsi,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#28
# asm 2: movq <mulrax=%rax,>mulx319_stack=216(%rsp)
movq %rax,216(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   rz0 = mulrax
# asm 1: mov  <mulrax=int64#7,>rz0=int64#2
# asm 2: mov  <mulrax=%rax,>rz0=%rsi
mov  %rax,%rsi

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#4
# asm 2: mov  <mulrdx=%rdx,>mulr01=%rcx
mov  %rdx,%rcx

# qhasm:   mulrax = g4_stack
# asm 1: movq <g4_stack=stack64#22,>mulrax=int64#3
# asm 2: movq <g4_stack=168(%rsp),>mulrax=%rdx
movq 168(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#29
# asm 2: movq <mulrax=%rax,>mulx419_stack=224(%rsp)
movq %rax,224(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   carry? rz0 += mulrax
# asm 1: add  <mulrax=int64#7,<rz0=int64#2
# asm 2: add  <mulrax=%rax,<rz0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = g0_stack
# asm 1: movq <g0_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <g0_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rz0 += mulrax
# asm 1: add  <mulrax=int64#7,<rz0=int64#2
# asm 2: add  <mulrax=%rax,<rz0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = g0_stack
# asm 1: movq <g0_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <g0_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   rz1 = mulrax
# asm 1: mov  <mulrax=int64#7,>rz1=int64#5
# asm 2: mov  <mulrax=%rax,>rz1=%r8
mov  %rax,%r8

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#6
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r9
mov  %rdx,%r9

# qhasm:   mulrax = g0_stack
# asm 1: movq <g0_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <g0_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   rz2 = mulrax
# asm 1: mov  <mulrax=int64#7,>rz2=int64#8
# asm 2: mov  <mulrax=%rax,>rz2=%r10
mov  %rax,%r10

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#9
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r11
mov  %rdx,%r11

# qhasm:   mulrax = g0_stack
# asm 1: movq <g0_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <g0_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   rz3 = mulrax
# asm 1: mov  <mulrax=int64#7,>rz3=int64#10
# asm 2: mov  <mulrax=%rax,>rz3=%r12
mov  %rax,%r12

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#11
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r13
mov  %rdx,%r13

# qhasm:   mulrax = g0_stack
# asm 1: movq <g0_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <g0_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   rz4 = mulrax
# asm 1: mov  <mulrax=int64#7,>rz4=int64#12
# asm 2: mov  <mulrax=%rax,>rz4=%r14
mov  %rax,%r14

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#13
# asm 2: mov  <mulrdx=%rdx,>mulr41=%r15
mov  %rdx,%r15

# qhasm:   mulrax = g1_stack
# asm 1: movq <g1_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <g1_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rz1 += mulrax
# asm 1: add  <mulrax=int64#7,<rz1=int64#5
# asm 2: add  <mulrax=%rax,<rz1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = g1_stack
# asm 1: movq <g1_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <g1_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   carry? rz2 += mulrax
# asm 1: add  <mulrax=int64#7,<rz2=int64#8
# asm 2: add  <mulrax=%rax,<rz2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = g1_stack
# asm 1: movq <g1_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <g1_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   carry? rz3 += mulrax
# asm 1: add  <mulrax=int64#7,<rz3=int64#10
# asm 2: add  <mulrax=%rax,<rz3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = g1_stack
# asm 1: movq <g1_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <g1_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   carry? rz4 += mulrax
# asm 1: add  <mulrax=int64#7,<rz4=int64#12
# asm 2: add  <mulrax=%rax,<rz4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = g1_stack
# asm 1: movq <g1_stack=stack64#19,>mulrax=int64#3
# asm 2: movq <g1_stack=144(%rsp),>mulrax=%rdx
movq 144(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   carry? rz0 += mulrax
# asm 1: add  <mulrax=int64#7,<rz0=int64#2
# asm 2: add  <mulrax=%rax,<rz0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = g2_stack
# asm 1: movq <g2_stack=stack64#20,>mulrax=int64#7
# asm 2: movq <g2_stack=152(%rsp),>mulrax=%rax
movq 152(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rz2 += mulrax
# asm 1: add  <mulrax=int64#7,<rz2=int64#8
# asm 2: add  <mulrax=%rax,<rz2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = g2_stack
# asm 1: movq <g2_stack=stack64#20,>mulrax=int64#7
# asm 2: movq <g2_stack=152(%rsp),>mulrax=%rax
movq 152(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   carry? rz3 += mulrax
# asm 1: add  <mulrax=int64#7,<rz3=int64#10
# asm 2: add  <mulrax=%rax,<rz3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = g2_stack
# asm 1: movq <g2_stack=stack64#20,>mulrax=int64#7
# asm 2: movq <g2_stack=152(%rsp),>mulrax=%rax
movq 152(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   carry? rz4 += mulrax
# asm 1: add  <mulrax=int64#7,<rz4=int64#12
# asm 2: add  <mulrax=%rax,<rz4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = g2_stack
# asm 1: movq <g2_stack=stack64#20,>mulrax=int64#3
# asm 2: movq <g2_stack=152(%rsp),>mulrax=%rdx
movq 152(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   carry? rz0 += mulrax
# asm 1: add  <mulrax=int64#7,<rz0=int64#2
# asm 2: add  <mulrax=%rax,<rz0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = g2_stack
# asm 1: movq <g2_stack=stack64#20,>mulrax=int64#3
# asm 2: movq <g2_stack=152(%rsp),>mulrax=%rdx
movq 152(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   carry? rz1 += mulrax
# asm 1: add  <mulrax=int64#7,<rz1=int64#5
# asm 2: add  <mulrax=%rax,<rz1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = g3_stack
# asm 1: movq <g3_stack=stack64#21,>mulrax=int64#7
# asm 2: movq <g3_stack=160(%rsp),>mulrax=%rax
movq 160(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rz3 += mulrax
# asm 1: add  <mulrax=int64#7,<rz3=int64#10
# asm 2: add  <mulrax=%rax,<rz3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = g3_stack
# asm 1: movq <g3_stack=stack64#21,>mulrax=int64#7
# asm 2: movq <g3_stack=160(%rsp),>mulrax=%rax
movq 160(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f1_stack
# asm 1: mulq  <f1_stack=stack64#24
# asm 2: mulq  <f1_stack=184(%rsp)
mulq  184(%rsp)

# qhasm:   carry? rz4 += mulrax
# asm 1: add  <mulrax=int64#7,<rz4=int64#12
# asm 2: add  <mulrax=%rax,<rz4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#28,>mulrax=int64#7
# asm 2: movq <mulx319_stack=216(%rsp),>mulrax=%rax
movq 216(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   carry? rz1 += mulrax
# asm 1: add  <mulrax=int64#7,<rz1=int64#5
# asm 2: add  <mulrax=%rax,<rz1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#28,>mulrax=int64#7
# asm 2: movq <mulx319_stack=216(%rsp),>mulrax=%rax
movq 216(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   carry? rz2 += mulrax
# asm 1: add  <mulrax=int64#7,<rz2=int64#8
# asm 2: add  <mulrax=%rax,<rz2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = g4_stack
# asm 1: movq <g4_stack=stack64#22,>mulrax=int64#7
# asm 2: movq <g4_stack=168(%rsp),>mulrax=%rax
movq 168(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f0_stack
# asm 1: mulq  <f0_stack=stack64#23
# asm 2: mulq  <f0_stack=176(%rsp)
mulq  176(%rsp)

# qhasm:   carry? rz4 += mulrax
# asm 1: add  <mulrax=int64#7,<rz4=int64#12
# asm 2: add  <mulrax=%rax,<rz4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#29,>mulrax=int64#7
# asm 2: movq <mulx419_stack=224(%rsp),>mulrax=%rax
movq 224(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f2_stack
# asm 1: mulq  <f2_stack=stack64#25
# asm 2: mulq  <f2_stack=192(%rsp)
mulq  192(%rsp)

# qhasm:   carry? rz1 += mulrax
# asm 1: add  <mulrax=int64#7,<rz1=int64#5
# asm 2: add  <mulrax=%rax,<rz1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#29,>mulrax=int64#7
# asm 2: movq <mulx419_stack=224(%rsp),>mulrax=%rax
movq 224(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f3_stack
# asm 1: mulq  <f3_stack=stack64#26
# asm 2: mulq  <f3_stack=200(%rsp)
mulq  200(%rsp)

# qhasm:   carry? rz2 += mulrax
# asm 1: add  <mulrax=int64#7,<rz2=int64#8
# asm 2: add  <mulrax=%rax,<rz2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#29,>mulrax=int64#7
# asm 2: movq <mulx419_stack=224(%rsp),>mulrax=%rax
movq 224(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * f4_stack
# asm 1: mulq  <f4_stack=stack64#27
# asm 2: mulq  <f4_stack=208(%rsp)
mulq  208(%rsp)

# qhasm:   carry? rz3 += mulrax
# asm 1: add  <mulrax=int64#7,<rz3=int64#10
# asm 2: add  <mulrax=%rax,<rz3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#3
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rdx
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rdx

# qhasm:   mulr01 = (mulr01.rz0) << 13
# asm 1: shld $13,<rz0=int64#2,<mulr01=int64#4
# asm 2: shld $13,<rz0=%rsi,<mulr01=%rcx
shld $13,%rsi,%rcx

# qhasm:   rz0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz0=int64#2
# asm 2: and  <mulredmask=%rdx,<rz0=%rsi
and  %rdx,%rsi

# qhasm:   mulr11 = (mulr11.rz1) << 13
# asm 1: shld $13,<rz1=int64#5,<mulr11=int64#6
# asm 2: shld $13,<rz1=%r8,<mulr11=%r9
shld $13,%r8,%r9

# qhasm:   rz1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz1=int64#5
# asm 2: and  <mulredmask=%rdx,<rz1=%r8
and  %rdx,%r8

# qhasm:   rz1 += mulr01
# asm 1: add  <mulr01=int64#4,<rz1=int64#5
# asm 2: add  <mulr01=%rcx,<rz1=%r8
add  %rcx,%r8

# qhasm:   mulr21 = (mulr21.rz2) << 13
# asm 1: shld $13,<rz2=int64#8,<mulr21=int64#9
# asm 2: shld $13,<rz2=%r10,<mulr21=%r11
shld $13,%r10,%r11

# qhasm:   rz2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz2=int64#8
# asm 2: and  <mulredmask=%rdx,<rz2=%r10
and  %rdx,%r10

# qhasm:   rz2 += mulr11
# asm 1: add  <mulr11=int64#6,<rz2=int64#8
# asm 2: add  <mulr11=%r9,<rz2=%r10
add  %r9,%r10

# qhasm:   mulr31 = (mulr31.rz3) << 13
# asm 1: shld $13,<rz3=int64#10,<mulr31=int64#11
# asm 2: shld $13,<rz3=%r12,<mulr31=%r13
shld $13,%r12,%r13

# qhasm:   rz3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz3=int64#10
# asm 2: and  <mulredmask=%rdx,<rz3=%r12
and  %rdx,%r12

# qhasm:   rz3 += mulr21
# asm 1: add  <mulr21=int64#9,<rz3=int64#10
# asm 2: add  <mulr21=%r11,<rz3=%r12
add  %r11,%r12

# qhasm:   mulr41 = (mulr41.rz4) << 13
# asm 1: shld $13,<rz4=int64#12,<mulr41=int64#13
# asm 2: shld $13,<rz4=%r14,<mulr41=%r15
shld $13,%r14,%r15

# qhasm:   rz4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz4=int64#12
# asm 2: and  <mulredmask=%rdx,<rz4=%r14
and  %rdx,%r14

# qhasm:   rz4 += mulr31
# asm 1: add  <mulr31=int64#11,<rz4=int64#12
# asm 2: add  <mulr31=%r13,<rz4=%r14
add  %r13,%r14

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#13,>mulr41=int64#4
# asm 2: imulq  $19,<mulr41=%r15,>mulr41=%rcx
imulq  $19,%r15,%rcx

# qhasm:   rz0 += mulr41
# asm 1: add  <mulr41=int64#4,<rz0=int64#2
# asm 2: add  <mulr41=%rcx,<rz0=%rsi
add  %rcx,%rsi

# qhasm:   mult = rz0
# asm 1: mov  <rz0=int64#2,>mult=int64#4
# asm 2: mov  <rz0=%rsi,>mult=%rcx
mov  %rsi,%rcx

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   mult += rz1
# asm 1: add  <rz1=int64#5,<mult=int64#4
# asm 2: add  <rz1=%r8,<mult=%rcx
add  %r8,%rcx

# qhasm:   rz1 = mult
# asm 1: mov  <mult=int64#4,>rz1=int64#5
# asm 2: mov  <mult=%rcx,>rz1=%r8
mov  %rcx,%r8

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rz0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz0=int64#2
# asm 2: and  <mulredmask=%rdx,<rz0=%rsi
and  %rdx,%rsi

# qhasm:   mult += rz2
# asm 1: add  <rz2=int64#8,<mult=int64#4
# asm 2: add  <rz2=%r10,<mult=%rcx
add  %r10,%rcx

# qhasm:   rz2 = mult
# asm 1: mov  <mult=int64#4,>rz2=int64#6
# asm 2: mov  <mult=%rcx,>rz2=%r9
mov  %rcx,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rz1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz1=int64#5
# asm 2: and  <mulredmask=%rdx,<rz1=%r8
and  %rdx,%r8

# qhasm:   mult += rz3
# asm 1: add  <rz3=int64#10,<mult=int64#4
# asm 2: add  <rz3=%r12,<mult=%rcx
add  %r12,%rcx

# qhasm:   rz3 = mult
# asm 1: mov  <mult=int64#4,>rz3=int64#7
# asm 2: mov  <mult=%rcx,>rz3=%rax
mov  %rcx,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rz2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz2=int64#6
# asm 2: and  <mulredmask=%rdx,<rz2=%r9
and  %rdx,%r9

# qhasm:   mult += rz4
# asm 1: add  <rz4=int64#12,<mult=int64#4
# asm 2: add  <rz4=%r14,<mult=%rcx
add  %r14,%rcx

# qhasm:   rz4 = mult
# asm 1: mov  <mult=int64#4,>rz4=int64#8
# asm 2: mov  <mult=%rcx,>rz4=%r10
mov  %rcx,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rz3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz3=int64#7
# asm 2: and  <mulredmask=%rdx,<rz3=%rax
and  %rdx,%rax

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#4,>mult=int64#4
# asm 2: imulq  $19,<mult=%rcx,>mult=%rcx
imulq  $19,%rcx,%rcx

# qhasm:   rz0 += mult
# asm 1: add  <mult=int64#4,<rz0=int64#2
# asm 2: add  <mult=%rcx,<rz0=%rsi
add  %rcx,%rsi

# qhasm:   rz4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rz4=int64#8
# asm 2: and  <mulredmask=%rdx,<rz4=%r10
and  %rdx,%r10

# qhasm: *(uint64 *)(rp + 80) = rz0
# asm 1: movq   <rz0=int64#2,80(<rp=int64#1)
# asm 2: movq   <rz0=%rsi,80(<rp=%rdi)
movq   %rsi,80(%rdi)

# qhasm: *(uint64 *)(rp + 88) = rz1
# asm 1: movq   <rz1=int64#5,88(<rp=int64#1)
# asm 2: movq   <rz1=%r8,88(<rp=%rdi)
movq   %r8,88(%rdi)

# qhasm: *(uint64 *)(rp + 96) = rz2
# asm 1: movq   <rz2=int64#6,96(<rp=int64#1)
# asm 2: movq   <rz2=%r9,96(<rp=%rdi)
movq   %r9,96(%rdi)

# qhasm: *(uint64 *)(rp + 104) = rz3
# asm 1: movq   <rz3=int64#7,104(<rp=int64#1)
# asm 2: movq   <rz3=%rax,104(<rp=%rdi)
movq   %rax,104(%rdi)

# qhasm: *(uint64 *)(rp + 112) = rz4
# asm 1: movq   <rz4=int64#8,112(<rp=int64#1)
# asm 2: movq   <rz4=%r10,112(<rp=%rdi)
movq   %r10,112(%rdi)

# qhasm:   mulrax = e3_stack
# asm 1: movq <e3_stack=stack64#16,>mulrax=int64#2
# asm 2: movq <e3_stack=120(%rsp),>mulrax=%rsi
movq 120(%rsp),%rsi

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#2,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rsi,>mulrax=%rax
imulq  $19,%rsi,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#18
# asm 2: movq <mulrax=%rax,>mulx319_stack=136(%rsp)
movq %rax,136(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * h2_stack
# asm 1: mulq  <h2_stack=stack64#10
# asm 2: mulq  <h2_stack=72(%rsp)
mulq  72(%rsp)

# qhasm:   rt0 = mulrax
# asm 1: mov  <mulrax=int64#7,>rt0=int64#2
# asm 2: mov  <mulrax=%rax,>rt0=%rsi
mov  %rax,%rsi

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#4
# asm 2: mov  <mulrdx=%rdx,>mulr01=%rcx
mov  %rdx,%rcx

# qhasm:   mulrax = e4_stack
# asm 1: movq <e4_stack=stack64#17,>mulrax=int64#3
# asm 2: movq <e4_stack=128(%rsp),>mulrax=%rdx
movq 128(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#19
# asm 2: movq <mulrax=%rax,>mulx419_stack=144(%rsp)
movq %rax,144(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * h1_stack
# asm 1: mulq  <h1_stack=stack64#9
# asm 2: mulq  <h1_stack=64(%rsp)
mulq  64(%rsp)

# qhasm:   carry? rt0 += mulrax
# asm 1: add  <mulrax=int64#7,<rt0=int64#2
# asm 2: add  <mulrax=%rax,<rt0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h0_stack
# asm 1: mulq  <h0_stack=stack64#8
# asm 2: mulq  <h0_stack=56(%rsp)
mulq  56(%rsp)

# qhasm:   carry? rt0 += mulrax
# asm 1: add  <mulrax=int64#7,<rt0=int64#2
# asm 2: add  <mulrax=%rax,<rt0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h1_stack
# asm 1: mulq  <h1_stack=stack64#9
# asm 2: mulq  <h1_stack=64(%rsp)
mulq  64(%rsp)

# qhasm:   rt1 = mulrax
# asm 1: mov  <mulrax=int64#7,>rt1=int64#5
# asm 2: mov  <mulrax=%rax,>rt1=%r8
mov  %rax,%r8

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#6
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r9
mov  %rdx,%r9

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h2_stack
# asm 1: mulq  <h2_stack=stack64#10
# asm 2: mulq  <h2_stack=72(%rsp)
mulq  72(%rsp)

# qhasm:   rt2 = mulrax
# asm 1: mov  <mulrax=int64#7,>rt2=int64#8
# asm 2: mov  <mulrax=%rax,>rt2=%r10
mov  %rax,%r10

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#9
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r11
mov  %rdx,%r11

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h3_stack
# asm 1: mulq  <h3_stack=stack64#11
# asm 2: mulq  <h3_stack=80(%rsp)
mulq  80(%rsp)

# qhasm:   rt3 = mulrax
# asm 1: mov  <mulrax=int64#7,>rt3=int64#10
# asm 2: mov  <mulrax=%rax,>rt3=%r12
mov  %rax,%r12

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#11
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r13
mov  %rdx,%r13

# qhasm:   mulrax = e0_stack
# asm 1: movq <e0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <e0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h4_stack
# asm 1: mulq  <h4_stack=stack64#12
# asm 2: mulq  <h4_stack=88(%rsp)
mulq  88(%rsp)

# qhasm:   rt4 = mulrax
# asm 1: mov  <mulrax=int64#7,>rt4=int64#12
# asm 2: mov  <mulrax=%rax,>rt4=%r14
mov  %rax,%r14

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#13
# asm 2: mov  <mulrdx=%rdx,>mulr41=%r15
mov  %rdx,%r15

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h0_stack
# asm 1: mulq  <h0_stack=stack64#8
# asm 2: mulq  <h0_stack=56(%rsp)
mulq  56(%rsp)

# qhasm:   carry? rt1 += mulrax
# asm 1: add  <mulrax=int64#7,<rt1=int64#5
# asm 2: add  <mulrax=%rax,<rt1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h1_stack
# asm 1: mulq  <h1_stack=stack64#9
# asm 2: mulq  <h1_stack=64(%rsp)
mulq  64(%rsp)

# qhasm:   carry? rt2 += mulrax
# asm 1: add  <mulrax=int64#7,<rt2=int64#8
# asm 2: add  <mulrax=%rax,<rt2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h2_stack
# asm 1: mulq  <h2_stack=stack64#10
# asm 2: mulq  <h2_stack=72(%rsp)
mulq  72(%rsp)

# qhasm:   carry? rt3 += mulrax
# asm 1: add  <mulrax=int64#7,<rt3=int64#10
# asm 2: add  <mulrax=%rax,<rt3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h3_stack
# asm 1: mulq  <h3_stack=stack64#11
# asm 2: mulq  <h3_stack=80(%rsp)
mulq  80(%rsp)

# qhasm:   carry? rt4 += mulrax
# asm 1: add  <mulrax=int64#7,<rt4=int64#12
# asm 2: add  <mulrax=%rax,<rt4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = e1_stack
# asm 1: movq <e1_stack=stack64#14,>mulrax=int64#3
# asm 2: movq <e1_stack=104(%rsp),>mulrax=%rdx
movq 104(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h4_stack
# asm 1: mulq  <h4_stack=stack64#12
# asm 2: mulq  <h4_stack=88(%rsp)
mulq  88(%rsp)

# qhasm:   carry? rt0 += mulrax
# asm 1: add  <mulrax=int64#7,<rt0=int64#2
# asm 2: add  <mulrax=%rax,<rt0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h0_stack
# asm 1: mulq  <h0_stack=stack64#8
# asm 2: mulq  <h0_stack=56(%rsp)
mulq  56(%rsp)

# qhasm:   carry? rt2 += mulrax
# asm 1: add  <mulrax=int64#7,<rt2=int64#8
# asm 2: add  <mulrax=%rax,<rt2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h1_stack
# asm 1: mulq  <h1_stack=stack64#9
# asm 2: mulq  <h1_stack=64(%rsp)
mulq  64(%rsp)

# qhasm:   carry? rt3 += mulrax
# asm 1: add  <mulrax=int64#7,<rt3=int64#10
# asm 2: add  <mulrax=%rax,<rt3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h2_stack
# asm 1: mulq  <h2_stack=stack64#10
# asm 2: mulq  <h2_stack=72(%rsp)
mulq  72(%rsp)

# qhasm:   carry? rt4 += mulrax
# asm 1: add  <mulrax=int64#7,<rt4=int64#12
# asm 2: add  <mulrax=%rax,<rt4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#3
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rdx
movq 112(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h3_stack
# asm 1: mulq  <h3_stack=stack64#11
# asm 2: mulq  <h3_stack=80(%rsp)
mulq  80(%rsp)

# qhasm:   carry? rt0 += mulrax
# asm 1: add  <mulrax=int64#7,<rt0=int64#2
# asm 2: add  <mulrax=%rax,<rt0=%rsi
add  %rax,%rsi

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#4
# asm 2: adc <mulrdx=%rdx,<mulr01=%rcx
adc %rdx,%rcx

# qhasm:   mulrax = e2_stack
# asm 1: movq <e2_stack=stack64#15,>mulrax=int64#3
# asm 2: movq <e2_stack=112(%rsp),>mulrax=%rdx
movq 112(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h4_stack
# asm 1: mulq  <h4_stack=stack64#12
# asm 2: mulq  <h4_stack=88(%rsp)
mulq  88(%rsp)

# qhasm:   carry? rt1 += mulrax
# asm 1: add  <mulrax=int64#7,<rt1=int64#5
# asm 2: add  <mulrax=%rax,<rt1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = e3_stack
# asm 1: movq <e3_stack=stack64#16,>mulrax=int64#7
# asm 2: movq <e3_stack=120(%rsp),>mulrax=%rax
movq 120(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h0_stack
# asm 1: mulq  <h0_stack=stack64#8
# asm 2: mulq  <h0_stack=56(%rsp)
mulq  56(%rsp)

# qhasm:   carry? rt3 += mulrax
# asm 1: add  <mulrax=int64#7,<rt3=int64#10
# asm 2: add  <mulrax=%rax,<rt3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulrax = e3_stack
# asm 1: movq <e3_stack=stack64#16,>mulrax=int64#7
# asm 2: movq <e3_stack=120(%rsp),>mulrax=%rax
movq 120(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h1_stack
# asm 1: mulq  <h1_stack=stack64#9
# asm 2: mulq  <h1_stack=64(%rsp)
mulq  64(%rsp)

# qhasm:   carry? rt4 += mulrax
# asm 1: add  <mulrax=int64#7,<rt4=int64#12
# asm 2: add  <mulrax=%rax,<rt4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h3_stack
# asm 1: mulq  <h3_stack=stack64#11
# asm 2: mulq  <h3_stack=80(%rsp)
mulq  80(%rsp)

# qhasm:   carry? rt1 += mulrax
# asm 1: add  <mulrax=int64#7,<rt1=int64#5
# asm 2: add  <mulrax=%rax,<rt1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h4_stack
# asm 1: mulq  <h4_stack=stack64#12
# asm 2: mulq  <h4_stack=88(%rsp)
mulq  88(%rsp)

# qhasm:   carry? rt2 += mulrax
# asm 1: add  <mulrax=int64#7,<rt2=int64#8
# asm 2: add  <mulrax=%rax,<rt2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = e4_stack
# asm 1: movq <e4_stack=stack64#17,>mulrax=int64#7
# asm 2: movq <e4_stack=128(%rsp),>mulrax=%rax
movq 128(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h0_stack
# asm 1: mulq  <h0_stack=stack64#8
# asm 2: mulq  <h0_stack=56(%rsp)
mulq  56(%rsp)

# qhasm:   carry? rt4 += mulrax
# asm 1: add  <mulrax=int64#7,<rt4=int64#12
# asm 2: add  <mulrax=%rax,<rt4=%r14
add  %rax,%r14

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr41=%r15
adc %rdx,%r15

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h2_stack
# asm 1: mulq  <h2_stack=stack64#10
# asm 2: mulq  <h2_stack=72(%rsp)
mulq  72(%rsp)

# qhasm:   carry? rt1 += mulrax
# asm 1: add  <mulrax=int64#7,<rt1=int64#5
# asm 2: add  <mulrax=%rax,<rt1=%r8
add  %rax,%r8

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr11=%r9
adc %rdx,%r9

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h3_stack
# asm 1: mulq  <h3_stack=stack64#11
# asm 2: mulq  <h3_stack=80(%rsp)
mulq  80(%rsp)

# qhasm:   carry? rt2 += mulrax
# asm 1: add  <mulrax=int64#7,<rt2=int64#8
# asm 2: add  <mulrax=%rax,<rt2=%r10
add  %rax,%r10

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr21=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * h4_stack
# asm 1: mulq  <h4_stack=stack64#12
# asm 2: mulq  <h4_stack=88(%rsp)
mulq  88(%rsp)

# qhasm:   carry? rt3 += mulrax
# asm 1: add  <mulrax=int64#7,<rt3=int64#10
# asm 2: add  <mulrax=%rax,<rt3=%r12
add  %rax,%r12

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr31=%r13
adc %rdx,%r13

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#3
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rdx
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rdx

# qhasm:   mulr01 = (mulr01.rt0) << 13
# asm 1: shld $13,<rt0=int64#2,<mulr01=int64#4
# asm 2: shld $13,<rt0=%rsi,<mulr01=%rcx
shld $13,%rsi,%rcx

# qhasm:   rt0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt0=int64#2
# asm 2: and  <mulredmask=%rdx,<rt0=%rsi
and  %rdx,%rsi

# qhasm:   mulr11 = (mulr11.rt1) << 13
# asm 1: shld $13,<rt1=int64#5,<mulr11=int64#6
# asm 2: shld $13,<rt1=%r8,<mulr11=%r9
shld $13,%r8,%r9

# qhasm:   rt1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt1=int64#5
# asm 2: and  <mulredmask=%rdx,<rt1=%r8
and  %rdx,%r8

# qhasm:   rt1 += mulr01
# asm 1: add  <mulr01=int64#4,<rt1=int64#5
# asm 2: add  <mulr01=%rcx,<rt1=%r8
add  %rcx,%r8

# qhasm:   mulr21 = (mulr21.rt2) << 13
# asm 1: shld $13,<rt2=int64#8,<mulr21=int64#9
# asm 2: shld $13,<rt2=%r10,<mulr21=%r11
shld $13,%r10,%r11

# qhasm:   rt2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt2=int64#8
# asm 2: and  <mulredmask=%rdx,<rt2=%r10
and  %rdx,%r10

# qhasm:   rt2 += mulr11
# asm 1: add  <mulr11=int64#6,<rt2=int64#8
# asm 2: add  <mulr11=%r9,<rt2=%r10
add  %r9,%r10

# qhasm:   mulr31 = (mulr31.rt3) << 13
# asm 1: shld $13,<rt3=int64#10,<mulr31=int64#11
# asm 2: shld $13,<rt3=%r12,<mulr31=%r13
shld $13,%r12,%r13

# qhasm:   rt3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt3=int64#10
# asm 2: and  <mulredmask=%rdx,<rt3=%r12
and  %rdx,%r12

# qhasm:   rt3 += mulr21
# asm 1: add  <mulr21=int64#9,<rt3=int64#10
# asm 2: add  <mulr21=%r11,<rt3=%r12
add  %r11,%r12

# qhasm:   mulr41 = (mulr41.rt4) << 13
# asm 1: shld $13,<rt4=int64#12,<mulr41=int64#13
# asm 2: shld $13,<rt4=%r14,<mulr41=%r15
shld $13,%r14,%r15

# qhasm:   rt4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt4=int64#12
# asm 2: and  <mulredmask=%rdx,<rt4=%r14
and  %rdx,%r14

# qhasm:   rt4 += mulr31
# asm 1: add  <mulr31=int64#11,<rt4=int64#12
# asm 2: add  <mulr31=%r13,<rt4=%r14
add  %r13,%r14

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#13,>mulr41=int64#4
# asm 2: imulq  $19,<mulr41=%r15,>mulr41=%rcx
imulq  $19,%r15,%rcx

# qhasm:   rt0 += mulr41
# asm 1: add  <mulr41=int64#4,<rt0=int64#2
# asm 2: add  <mulr41=%rcx,<rt0=%rsi
add  %rcx,%rsi

# qhasm:   mult = rt0
# asm 1: mov  <rt0=int64#2,>mult=int64#4
# asm 2: mov  <rt0=%rsi,>mult=%rcx
mov  %rsi,%rcx

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   mult += rt1
# asm 1: add  <rt1=int64#5,<mult=int64#4
# asm 2: add  <rt1=%r8,<mult=%rcx
add  %r8,%rcx

# qhasm:   rt1 = mult
# asm 1: mov  <mult=int64#4,>rt1=int64#5
# asm 2: mov  <mult=%rcx,>rt1=%r8
mov  %rcx,%r8

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rt0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt0=int64#2
# asm 2: and  <mulredmask=%rdx,<rt0=%rsi
and  %rdx,%rsi

# qhasm:   mult += rt2
# asm 1: add  <rt2=int64#8,<mult=int64#4
# asm 2: add  <rt2=%r10,<mult=%rcx
add  %r10,%rcx

# qhasm:   rt2 = mult
# asm 1: mov  <mult=int64#4,>rt2=int64#6
# asm 2: mov  <mult=%rcx,>rt2=%r9
mov  %rcx,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rt1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt1=int64#5
# asm 2: and  <mulredmask=%rdx,<rt1=%r8
and  %rdx,%r8

# qhasm:   mult += rt3
# asm 1: add  <rt3=int64#10,<mult=int64#4
# asm 2: add  <rt3=%r12,<mult=%rcx
add  %r12,%rcx

# qhasm:   rt3 = mult
# asm 1: mov  <mult=int64#4,>rt3=int64#7
# asm 2: mov  <mult=%rcx,>rt3=%rax
mov  %rcx,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rt2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt2=int64#6
# asm 2: and  <mulredmask=%rdx,<rt2=%r9
and  %rdx,%r9

# qhasm:   mult += rt4
# asm 1: add  <rt4=int64#12,<mult=int64#4
# asm 2: add  <rt4=%r14,<mult=%rcx
add  %r14,%rcx

# qhasm:   rt4 = mult
# asm 1: mov  <mult=int64#4,>rt4=int64#8
# asm 2: mov  <mult=%rcx,>rt4=%r10
mov  %rcx,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   rt3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt3=int64#7
# asm 2: and  <mulredmask=%rdx,<rt3=%rax
and  %rdx,%rax

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#4,>mult=int64#4
# asm 2: imulq  $19,<mult=%rcx,>mult=%rcx
imulq  $19,%rcx,%rcx

# qhasm:   rt0 += mult
# asm 1: add  <mult=int64#4,<rt0=int64#2
# asm 2: add  <mult=%rcx,<rt0=%rsi
add  %rcx,%rsi

# qhasm:   rt4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<rt4=int64#8
# asm 2: and  <mulredmask=%rdx,<rt4=%r10
and  %rdx,%r10

# qhasm: *(uint64 *)(rp + 120) = rt0
# asm 1: movq   <rt0=int64#2,120(<rp=int64#1)
# asm 2: movq   <rt0=%rsi,120(<rp=%rdi)
movq   %rsi,120(%rdi)

# qhasm: *(uint64 *)(rp + 128) = rt1
# asm 1: movq   <rt1=int64#5,128(<rp=int64#1)
# asm 2: movq   <rt1=%r8,128(<rp=%rdi)
movq   %r8,128(%rdi)

# qhasm: *(uint64 *)(rp + 136) = rt2
# asm 1: movq   <rt2=int64#6,136(<rp=int64#1)
# asm 2: movq   <rt2=%r9,136(<rp=%rdi)
movq   %r9,136(%rdi)

# qhasm: *(uint64 *)(rp + 144) = rt3
# asm 1: movq   <rt3=int64#7,144(<rp=int64#1)
# asm 2: movq   <rt3=%rax,144(<rp=%rdi)
movq   %rax,144(%rdi)

# qhasm: *(uint64 *)(rp + 152) = rt4
# asm 1: movq   <rt4=int64#8,152(<rp=int64#1)
# asm 2: movq   <rt4=%r10,152(<rp=%rdi)
movq   %r10,152(%rdi)

# qhasm:   caller1 = caller1_stack
# asm 1: movq <caller1_stack=stack64#1,>caller1=int64#9
# asm 2: movq <caller1_stack=0(%rsp),>caller1=%r11
movq 0(%rsp),%r11

# qhasm:   caller2 = caller2_stack
# asm 1: movq <caller2_stack=stack64#2,>caller2=int64#10
# asm 2: movq <caller2_stack=8(%rsp),>caller2=%r12
movq 8(%rsp),%r12

# qhasm:   caller3 = caller3_stack
# asm 1: movq <caller3_stack=stack64#3,>caller3=int64#11
# asm 2: movq <caller3_stack=16(%rsp),>caller3=%r13
movq 16(%rsp),%r13

# qhasm:   caller4 = caller4_stack
# asm 1: movq <caller4_stack=stack64#4,>caller4=int64#12
# asm 2: movq <caller4_stack=24(%rsp),>caller4=%r14
movq 24(%rsp),%r14

# qhasm:   caller5 = caller5_stack
# asm 1: movq <caller5_stack=stack64#5,>caller5=int64#13
# asm 2: movq <caller5_stack=32(%rsp),>caller5=%r15
movq 32(%rsp),%r15

# qhasm:   caller6 = caller6_stack
# asm 1: movq <caller6_stack=stack64#6,>caller6=int64#14
# asm 2: movq <caller6_stack=40(%rsp),>caller6=%rbx
movq 40(%rsp),%rbx

# qhasm:   caller7 = caller7_stack
# asm 1: movq <caller7_stack=stack64#7,>caller7=int64#15
# asm 2: movq <caller7_stack=48(%rsp),>caller7=%rbp
movq 48(%rsp),%rbp

# qhasm: leave
add %r11,%rsp
mov %rdi,%rax
mov %rsi,%rdx
ret
